@@ -201,27 +201,29 @@ class ResourceTracker(object):
     def rebuild_claim(self, context, instance, nodename, allocations,
                       limits=None, image_meta=None, migration=None):
         """Create a claim for a rebuild operation."""
-        instance_type = instance.flavor
         return self._move_claim(
-            context, instance, instance_type, nodename, migration, allocations,
-            move_type=fields.MigrationType.EVACUATION,
+            context, instance, instance.flavor, nodename, migration,
+            allocations, move_type=fields.MigrationType.EVACUATION,
             image_meta=image_meta, limits=limits)
 
     @utils.synchronized(COMPUTE_RESOURCE_SEMAPHORE, fair=True)
-    def resize_claim(self, context, instance, instance_type, nodename,
-                     migration, allocations, image_meta=None, limits=None):
+    def resize_claim(
+        self, context, instance, flavor, nodename, migration, allocations,
+        image_meta=None, limits=None,
+    ):
         """Create a claim for a resize or cold-migration move.
 
         Note that this code assumes ``instance.new_flavor`` is set when
         resizing with a new flavor.
         """
-        return self._move_claim(context, instance, instance_type, nodename,
-                                migration, allocations, image_meta=image_meta,
-                                limits=limits)
+        return self._move_claim(
+            context, instance, flavor, nodename, migration,
+            allocations, image_meta=image_meta, limits=limits)
 
     @utils.synchronized(COMPUTE_RESOURCE_SEMAPHORE, fair=True)
-    def live_migration_claim(self, context, instance, nodename, migration,
-                             limits, allocs):
+    def live_migration_claim(
+        self, context, instance, nodename, migration, limits, allocs,
+    ):
         """Builds a MoveClaim for a live migration.
 
         :param context: The request context.
@@ -235,17 +237,18 @@ class ResourceTracker(object):
         :returns: A MoveClaim for this live migration.
         """
         # Flavor and image cannot change during a live migration.
-        instance_type = instance.flavor
+        flavor = instance.flavor
         image_meta = instance.image_meta
         return self._move_claim(
-            context, instance, instance_type, nodename, migration, allocs,
+            context, instance, flavor, nodename, migration, allocs,
             move_type=fields.MigrationType.LIVE_MIGRATION,
             image_meta=image_meta, limits=limits,
         )
 
-    def _move_claim(self, context, instance, new_instance_type, nodename,
-                    migration, allocations, move_type=None,
-                    image_meta=None, limits=None):
+    def _move_claim(
+        self, context, instance, new_flavor, nodename, migration, allocations,
+        move_type=None, image_meta=None, limits=None,
+    ):
         """Indicate that resources are needed for a move to this host.
 
         Move can be either a migrate/resize, live-migrate or an
@@ -253,7 +256,7 @@ class ResourceTracker(object):
 
         :param context: security context
         :param instance: instance object to reserve resources for
-        :param new_instance_type: new instance_type being resized to
+        :param new_flavor: new flavor being resized to
         :param nodename: The Ironic nodename selected by the scheduler
         :param migration: A migration object if one was already created
                           elsewhere for this operation (otherwise None)
@@ -271,9 +274,8 @@ class ResourceTracker(object):
         if migration:
             self._claim_existing_migration(migration, nodename)
         else:
-            migration = self._create_migration(context, instance,
-                                               new_instance_type,
-                                               nodename, move_type)
+            migration = self._create_migration(
+                context, instance, new_flavor, nodename, move_type)
 
         if self.disabled(nodename):
             # compute_driver doesn't support resource tracking, just
@@ -287,7 +289,7 @@ class ResourceTracker(object):
         # the old/new pci device in the resize phase. In the future
         # we would like to optimise this.
         new_pci_requests = pci_request.get_pci_requests_from_flavor(
-            new_instance_type)
+            new_flavor)
         new_pci_requests.instance_uuid = instance.uuid
         # On resize merge the SR-IOV ports pci_requests
         # with the new instance flavor pci_requests.
@@ -296,7 +298,7 @@ class ResourceTracker(object):
                 if request.source == objects.InstancePCIRequest.NEUTRON_PORT:
                     new_pci_requests.requests.append(request)
         claim = claims.MoveClaim(context, instance, nodename,
-                                 new_instance_type, image_meta, self, cn,
+                                 new_flavor, image_meta, self, cn,
                                  new_pci_requests, migration, limits=limits)
 
         claimed_pci_devices_objs = []
@@ -326,7 +328,12 @@ class ResourceTracker(object):
             migration_id=migration.id,
             old_numa_topology=instance.numa_topology,
             new_numa_topology=claim.claimed_numa_topology,
-            old_pci_devices=instance.pci_devices,
+            # NOTE(gibi): the _update_usage_from_migration call below appends
+            # the newly claimed pci devices to the instance.pci_devices list
+            # to keep the migration context independent we need to make a copy
+            # that list here. We need a deep copy as we need to duplicate
+            # the instance.pci_devices.objects list
+            old_pci_devices=copy.deepcopy(instance.pci_devices),
             new_pci_devices=claimed_pci_devices,
             old_pci_requests=instance.pci_requests,
             new_pci_requests=new_pci_requests,
@@ -345,8 +352,9 @@ class ResourceTracker(object):
 
         return claim
 
-    def _create_migration(self, context, instance, new_instance_type,
-                          nodename, move_type=None):
+    def _create_migration(
+        self, context, instance, new_flavor, nodename, move_type=None,
+    ):
         """Create a migration record for the upcoming resize.  This should
         be done while the COMPUTE_RESOURCES_SEMAPHORE is held so the resource
         claim will not be lost if the audit process starts.
@@ -356,7 +364,7 @@ class ResourceTracker(object):
         migration.dest_node = nodename
         migration.dest_host = self.driver.get_host_ip_addr()
         migration.old_instance_type_id = instance.flavor.id
-        migration.new_instance_type_id = new_instance_type.id
+        migration.new_instance_type_id = new_flavor.id
         migration.status = 'pre-migrating'
         migration.instance_uuid = instance.uuid
         migration.source_compute = instance.host
@@ -587,38 +595,35 @@ class ResourceTracker(object):
 
     @utils.synchronized(COMPUTE_RESOURCE_SEMAPHORE, fair=True)
     def drop_move_claim(self, context, instance, nodename,
-                        instance_type=None, prefix='new_'):
+                        flavor=None, prefix='new_'):
         self._drop_move_claim(
-            context, instance, nodename, instance_type, prefix='new_')
+            context, instance, nodename, flavor, prefix='new_')
 
     def _drop_move_claim(
-        self, context, instance, nodename, instance_type=None, prefix='new_',
+        self, context, instance, nodename, flavor=None, prefix='new_',
     ):
         """Remove usage for an incoming/outgoing migration.
 
         :param context: Security context.
         :param instance: The instance whose usage is to be removed.
         :param nodename: Host on which to remove usage. If the migration
-                         completed successfully, this is normally the source.
-                         If it did not complete successfully (failed or
-                         reverted), this is normally the destination.
-        :param instance_type: The flavor that determines the usage to remove.
-                              If the migration completed successfully, this is
-                              the old flavor to be removed from the source. If
-                              the migration did not complete successfully, this
-                              is the new flavor to be removed from the
-                              destination.
+            completed successfully, this is normally the source.  If it did not
+            complete successfully (failed or reverted), this is normally the
+            destination.
+        :param flavor: The flavor that determines the usage to remove.  If the
+            migration completed successfully, this is the old flavor to be
+            removed from the source. If the migration did not complete
+            successfully, this is the new flavor to be removed from the
+            destination.
         :param prefix: Prefix to use when accessing migration context
-                       attributes. 'old_' or 'new_', with 'new_' being the
-                       default.
+            attributes. 'old_' or 'new_', with 'new_' being the default.
         """
         # Remove usage for an instance that is tracked in migrations, such as
         # on the dest node during revert resize.
         if instance['uuid'] in self.tracked_migrations:
             migration = self.tracked_migrations.pop(instance['uuid'])
-            if not instance_type:
-                instance_type = self._get_instance_type(instance, prefix,
-                                                        migration)
+            if not flavor:
+                flavor = self._get_flavor(instance, prefix, migration)
         # Remove usage for an instance that is not tracked in migrations (such
         # as on the source node after a migration).
         # NOTE(lbeliveau): On resize on the same node, the instance is
@@ -626,11 +631,11 @@ class ResourceTracker(object):
         elif instance['uuid'] in self.tracked_instances:
             self.tracked_instances.remove(instance['uuid'])
 
-        if instance_type is not None:
+        if flavor is not None:
             numa_topology = self._get_migration_context_resource(
                 'numa_topology', instance, prefix=prefix)
             usage = self._get_usage_dict(
-                    instance_type, instance, numa_topology=numa_topology)
+                    flavor, instance, numa_topology=numa_topology)
             self._drop_pci_devices(instance, nodename, prefix)
             resources = self._get_migration_context_resource(
                 'resources', instance, prefix=prefix)
@@ -763,8 +768,7 @@ class ResourceTracker(object):
 
     def _setup_pci_tracker(self, context, compute_node, resources):
         if not self.pci_tracker:
-            n_id = compute_node.id
-            self.pci_tracker = pci_manager.PciDevTracker(context, node_id=n_id)
+            self.pci_tracker = pci_manager.PciDevTracker(context, compute_node)
             if 'pci_passthrough_devices' in resources:
                 dev_json = resources.pop('pci_passthrough_devices')
                 self.pci_tracker.update_devices_from_hypervisor_resources(
@@ -979,18 +983,13 @@ class ResourceTracker(object):
                 context, self.compute_nodes[nodename], migrations,
                 instance_by_uuid)
 
-        # Detect and account for orphaned instances that may exist on the
-        # hypervisor, but are not in the DB:
-        orphans = self._find_orphaned_instances()
-        self._update_usage_from_orphans(orphans, nodename)
-
         cn = self.compute_nodes[nodename]
 
         # NOTE(yjiang5): Because pci device tracker status is not cleared in
         # this periodic task, and also because the resource tracker is not
         # notified when instances are deleted, we need remove all usages
         # from deleted instances.
-        self.pci_tracker.clean_usage(instances, migrations, orphans)
+        self.pci_tracker.clean_usage(instances, migrations)
         dev_pools_obj = self.pci_tracker.stats.to_device_pools_obj()
         cn.pci_device_pools = dev_pools_obj
 
@@ -1331,9 +1330,8 @@ class ResourceTracker(object):
         if same_node:
             # Same node resize. Record usage for the 'new_' resources.  This
             # is executed on resize_claim().
-            if (instance['instance_type_id'] ==
-                    migration.old_instance_type_id):
-                itype = self._get_instance_type(instance, 'new_', migration)
+            if instance['instance_type_id'] == migration.old_instance_type_id:
+                itype = self._get_flavor(instance, 'new_', migration)
                 numa_topology = self._get_migration_context_resource(
                     'numa_topology', instance)
                 # Allocate pci device(s) for the instance.
@@ -1349,13 +1347,13 @@ class ResourceTracker(object):
                 # _update_usage_from_instances().  This method will then be
                 # called, and we need to account for the '_old' resources
                 # (just in case).
-                itype = self._get_instance_type(instance, 'old_', migration)
+                itype = self._get_flavor(instance, 'old_', migration)
                 numa_topology = self._get_migration_context_resource(
                     'numa_topology', instance, prefix='old_')
 
         elif incoming and not tracked:
             # instance has not yet migrated here:
-            itype = self._get_instance_type(instance, 'new_', migration)
+            itype = self._get_flavor(instance, 'new_', migration)
             numa_topology = self._get_migration_context_resource(
                 'numa_topology', instance)
             # Allocate pci device(s) for the instance.
@@ -1365,7 +1363,7 @@ class ResourceTracker(object):
 
         elif outbound and not tracked:
             # instance migrated, but record usage for a possible revert:
-            itype = self._get_instance_type(instance, 'old_', migration)
+            itype = self._get_flavor(instance, 'old_', migration)
             numa_topology = self._get_migration_context_resource(
                 'numa_topology', instance, prefix='old_')
             # We could be racing with confirm_resize setting the
@@ -1608,8 +1606,12 @@ class ResourceTracker(object):
                           "Deleting allocations that remained for this "
                           "instance against this compute host: %s.",
                           instance_uuid, alloc)
+                # We don't force delete the allocation in this case because if
+                # there is a conflict we'll retry on the next
+                # update_available_resource periodic run.
                 self.reportclient.delete_allocation_for_instance(context,
-                                                                 instance_uuid)
+                                                                 instance_uuid,
+                                                                 force=False)
                 continue
             if not instance.host:
                 # Allocations related to instances being scheduled should not
@@ -1675,44 +1677,10 @@ class ResourceTracker(object):
                       "instance on the %s node %s",
                       node_type, cn_uuid, instance=instance)
 
-    def _find_orphaned_instances(self):
-        """Given the set of instances and migrations already account for
-        by resource tracker, sanity check the hypervisor to determine
-        if there are any "orphaned" instances left hanging around.
-
-        Orphans could be consuming memory and should be accounted for in
-        usage calculations to guard against potential out of memory
-        errors.
-        """
-        uuids1 = frozenset(self.tracked_instances)
-        uuids2 = frozenset(self.tracked_migrations.keys())
-        uuids = uuids1 | uuids2
-
-        usage = self.driver.get_per_instance_usage()
-        vuuids = frozenset(usage.keys())
-
-        orphan_uuids = vuuids - uuids
-        orphans = [usage[uuid] for uuid in orphan_uuids]
-
-        return orphans
-
-    def _update_usage_from_orphans(self, orphans, nodename):
-        """Include orphaned instances in usage."""
-        for orphan in orphans:
-            memory_mb = orphan['memory_mb']
-
-            LOG.warning("Detected running orphan instance: %(uuid)s "
-                        "(consuming %(memory_mb)s MB memory)",
-                        {'uuid': orphan['uuid'], 'memory_mb': memory_mb})
-
-            # just record memory usage for the orphan
-            usage = {'memory_mb': memory_mb}
-            self._update_usage(usage, nodename)
-
     def delete_allocation_for_shelve_offloaded_instance(self, context,
                                                         instance):
-        self.reportclient.delete_allocation_for_instance(context,
-                                                         instance.uuid)
+        self.reportclient.delete_allocation_for_instance(
+            context, instance.uuid, force=True)
 
     def _verify_resources(self, resources):
         resource_keys = ["vcpus", "memory_mb", "local_gb", "cpu_info",
@@ -1724,15 +1692,15 @@ class ResourceTracker(object):
             reason = _("Missing keys: %s") % missing_keys
             raise exception.InvalidInput(reason=reason)
 
-    def _get_instance_type(self, instance, prefix, migration):
-        """Get the instance type from instance."""
+    def _get_flavor(self, instance, prefix, migration):
+        """Get the flavor from instance."""
         if migration.is_resize:
             return getattr(instance, '%sflavor' % prefix)
-        else:
-            # NOTE(ndipanov): Certain migration types (all but resize)
-            # do not change flavors so there is no need to stash
-            # them. In that case - just get the instance flavor.
-            return instance.flavor
+
+        # NOTE(ndipanov): Certain migration types (all but resize)
+        # do not change flavors so there is no need to stash
+        # them. In that case - just get the instance flavor.
+        return instance.flavor
 
     def _get_usage_dict(self, object_or_dict, instance, **updates):
         """Make a usage dict _update methods expect.
@@ -1939,9 +1907,7 @@ class ResourceTracker(object):
         self.stats[nodename].build_succeeded()
 
     @utils.synchronized(COMPUTE_RESOURCE_SEMAPHORE, fair=True)
-    def claim_pci_devices(
-        self, context, pci_requests, instance_numa_topology=None
-    ):
+    def claim_pci_devices(self, context, pci_requests, instance_numa_topology):
         """Claim instance PCI resources
 
         :param context: security context
@@ -2015,3 +1981,21 @@ class ResourceTracker(object):
         if migration:
             migration.status = 'done'
             migration.save()
+
+    @utils.synchronized(COMPUTE_RESOURCE_SEMAPHORE, fair=True)
+    def clean_compute_node_cache(self, compute_nodes_in_db):
+        """Clean the compute node cache of any nodes that no longer exist.
+
+        :param compute_nodes_in_db: list of ComputeNode objects from the DB.
+        """
+        compute_nodes_in_db_nodenames = {cn.hypervisor_hostname
+                                         for cn in compute_nodes_in_db}
+        stale_cns = set(self.compute_nodes) - compute_nodes_in_db_nodenames
+
+        for stale_cn in stale_cns:
+            # NOTE(mgoddard): we have found a node in the cache that has no
+            # compute node in the DB. This could be due to a node rebalance
+            # where another compute service took ownership of the node. Clean
+            # up the cache.
+            self.remove_node(stale_cn)
+            self.reportclient.invalidate_resource_provider(stale_cn)
