@@ -270,62 +270,6 @@ Related options:
 
 * ``compute_driver`` (libvirt)
 * ``[libvirt]/images_type`` (rbd)
-"""),
-    # TODO(lyarwood): Remove this workaround in the W release once all
-    # supported distros have rebased to a version of libgcrypt that does not
-    # have the performance issues listed below.
-    cfg.BoolOpt(
-        'disable_native_luksv1',
-        default=False,
-        help="""
-When attaching encrypted LUKSv1 Cinder volumes to instances the Libvirt driver
-configures the encrypted disks to be natively decrypted by QEMU.
-
-A performance issue has been discovered in the libgcrypt library used by QEMU
-that serverly limits the I/O performance in this scenario.
-
-For more information please refer to the following bug report:
-
-RFE: hardware accelerated AES-XTS mode
-https://bugzilla.redhat.com/show_bug.cgi?id=1762765
-
-Enabling this workaround option will cause Nova to use the legacy dm-crypt
-based os-brick encryptor to decrypt the LUKSv1 volume.
-
-Note that enabling this option while using volumes that do not provide a host
-block device such as Ceph will result in a failure to boot from or attach the
-volume to an instance. See the ``[workarounds]/rbd_block_device`` option for a
-way to avoid this for RBD.
-
-Related options:
-
-* ``compute_driver`` (libvirt)
-* ``rbd_block_device`` (workarounds)
-"""),
-    # TODO(lyarwood): Remove this workaround in the W release when the
-    # above disable_native_luksv1 configurable is removed.
-    cfg.BoolOpt('rbd_volume_local_attach',
-                default=False,
-                help="""
-Attach RBD Cinder volumes to the compute as host block devices.
-
-When enabled this option instructs os-brick to connect RBD volumes locally on
-the compute host as block devices instead of natively through QEMU.
-
-This workaround does not currently support extending attached volumes.
-
-This can be used with the disable_native_luksv1 workaround configuration
-option to avoid the recently discovered performance issues found within the
-libgcrypt library.
-
-This workaround is temporary and will be removed during the W release once
-all impacted distributions have been able to update their versions of the
-libgcrypt library.
-
-Related options:
-
-* ``compute_driver`` (libvirt)
-* ``disable_qemu_native_luksv1`` (workarounds)
 """),
     cfg.BoolOpt('reserve_disk_resource_for_image_cache',
                default=False,
@@ -345,6 +289,16 @@ Related options:
 * :oslo.config:option:`DEFAULT.instances_path`
 * :oslo.config:option:`image_cache.subdirectory_name`
 * :oslo.config:option:`update_resources_interval`
+"""),
+    cfg.BoolOpt('libvirt_disable_apic',
+               default=False,
+               help="""
+With some kernels initializing the guest apic can result in a kernel hang that
+renders the guest unusable. This happens as a result of a kernel bug. In most
+cases the correct fix it to update the guest image kernel to one that is
+patched however in some cases this is not possible. This workaround allows the
+emulation of an apic to be disabled per host however it is not recommended to
+use outside of a CI or developer cloud.
 """),
     cfg.ListOpt('wait_for_vif_plugged_event_during_hard_reboot',
                 item_type=cfg.types.String(
@@ -356,6 +310,9 @@ Related options:
                         "direct-physical",
                         "virtio-forwarder",
                         "smart-nic",
+                        "vdpa",
+                        "accelerator-direct",
+                        "accelerator-direct-physical",
                     ]),
                 default=[],
                 help="""
@@ -376,6 +333,9 @@ during hard reboot. The possible values are neutron port vnic types:
 * direct-physical
 * virtio-forwarder
 * smart-nic
+* vdpa
+* accelerator-direct
+* accelerator-direct-physical
 
 Adding a ``vnic_type`` to this configuration makes Nova wait for a
 network-vif-plugged event for each of the instance's vifs having the specific
@@ -398,6 +358,29 @@ backend.
 Related options:
 
 * :oslo.config:option:`DEFAULT.vif_plugging_timeout`
+"""),
+    cfg.BoolOpt('enable_qemu_monitor_announce_self',
+                default=False,
+                help="""
+If it is set to True the libvirt driver will  try as a best effort to send
+the announce-self command to the QEMU monitor so that it generates RARP frames
+to update network switches in the post live migration phase on the destination.
+
+Please note that this causes the domain to be considered tainted by libvirt.
+
+Related options:
+
+* :oslo.config:option:`DEFAULT.compute_driver` (libvirt)
+"""),
+    cfg.BoolOpt('disable_compute_service_check_for_ffu',
+                default=False,
+                help="""
+If this is set, the normal safety check for old compute services will be
+treated as a warning instead of an error. This is only to be enabled to
+facilitate a Fast-Forward upgrade where new control services are being started
+before compute nodes have been able to update their service record. In an FFU,
+the service records in the database will be more than one version old until
+the compute nodes start up, but control services need to be online first.
 """),
 ]
 
