@@ -39,8 +39,6 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import excutils
 from oslo_utils import timeutils
-import six
-from six.moves import range
 
 import nova.conf
 from nova import exception
@@ -180,7 +178,8 @@ class GlanceClientWrapper(object):
         kwargs = kwargs or {}
         retry_excs = (glanceclient.exc.ServiceUnavailable,
                 glanceclient.exc.InvalidEndpoint,
-                glanceclient.exc.CommunicationError)
+                glanceclient.exc.CommunicationError,
+                IOError)
         num_attempts = 1 + CONF.glance.num_retries
         controller_name = controller or 'images'
 
@@ -208,7 +207,7 @@ class GlanceClientWrapper(object):
                                'method': method, 'extra': extra})
                 if attempt == num_attempts:
                     raise exception.GlanceConnectionFailed(
-                        server=str(self.api_server), reason=six.text_type(e))
+                        server=str(self.api_server), reason=str(e))
                 time.sleep(1)
 
 
@@ -347,11 +346,7 @@ class GlanceImageServiceV2(object):
         """Calls out to Glance for data and writes data."""
 
         # First, check if image could be directly downloaded by special handler
-        # TODO(stephenfin): Remove check for 'allowed_direct_url_schemes' when
-        # we clean up tests since it's not used elsewhere
-        if ((CONF.glance.allowed_direct_url_schemes or
-             self._download_handlers) and dst_path is not None
-        ):
+        if (self._download_handlers and dst_path is not None):
             image = self.show(context, image_id, include_locations=True)
             for entry in image.get('locations', []):
                 loc_url = entry['url']
@@ -730,7 +725,7 @@ class GlanceImageServiceV2(object):
         except glanceclient.exc.HTTPForbidden:
             raise exception.ImageNotAuthorized(image_id=image_id)
         except glanceclient.exc.HTTPConflict as exc:
-            raise exception.ImageDeleteConflict(reason=six.text_type(exc))
+            raise exception.ImageDeleteConflict(reason=str(exc))
         return True
 
     def image_import_copy(self, context, image_id, stores):
@@ -873,8 +868,7 @@ def _convert_to_v2(image_meta):
                 # in glance only string and None property values are allowed,
                 # v1 client accepts any values and converts them to string,
                 # v2 doesn't - so we have to take care of it.
-                elif prop_value is None or isinstance(
-                        prop_value, six.string_types):
+                elif prop_value is None or isinstance(prop_value, str):
                     output[prop_name] = prop_value
                 else:
                     output[prop_name] = str(prop_value)
@@ -910,13 +904,13 @@ def _convert_timestamps_to_datetimes(image_meta):
 # NOTE(bcwaldon): used to store non-string data in glance metadata
 def _json_loads(properties, attr):
     prop = properties[attr]
-    if isinstance(prop, six.string_types):
+    if isinstance(prop, str):
         properties[attr] = jsonutils.loads(prop)
 
 
 def _json_dumps(properties, attr):
     prop = properties[attr]
-    if not isinstance(prop, six.string_types):
+    if not isinstance(prop, str):
         properties[attr] = jsonutils.dumps(prop)
 
 
@@ -1050,7 +1044,7 @@ def _translate_image_exception(image_id, exc_value):
         return exception.ImageNotFound(image_id=image_id)
     if isinstance(exc_value, glanceclient.exc.BadRequest):
         return exception.ImageBadRequest(image_id=image_id,
-                                         response=six.text_type(exc_value))
+                                         response=str(exc_value))
     if isinstance(exc_value, glanceclient.exc.HTTPOverLimit):
         return exception.ImageQuotaExceeded(image_id=image_id)
     return exc_value
@@ -1059,11 +1053,11 @@ def _translate_image_exception(image_id, exc_value):
 def _translate_plain_exception(exc_value):
     if isinstance(exc_value, (glanceclient.exc.Forbidden,
                     glanceclient.exc.Unauthorized)):
-        return exception.Forbidden(six.text_type(exc_value))
+        return exception.Forbidden(str(exc_value))
     if isinstance(exc_value, glanceclient.exc.NotFound):
-        return exception.NotFound(six.text_type(exc_value))
+        return exception.NotFound(str(exc_value))
     if isinstance(exc_value, glanceclient.exc.BadRequest):
-        return exception.Invalid(six.text_type(exc_value))
+        return exception.Invalid(str(exc_value))
     return exc_value
 
 
@@ -1081,7 +1075,7 @@ def _verify_certs(context, img_sig_cert_uuid, trusted_certs):
                     'failed for certificate: %s',
                     img_sig_cert_uuid)
         raise exception.CertificateValidationFailed(
-            cert_uuid=img_sig_cert_uuid, reason=six.text_type(e))
+            cert_uuid=img_sig_cert_uuid, reason=str(e))
 
 
 def get_remote_image_service(context, image_href):
