@@ -16,18 +16,13 @@ from oslo_db import exception as db_exc
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import uuidutils
-from sqlalchemy.orm import contains_eager
-from sqlalchemy.orm import joinedload
-from sqlalchemy.sql import func
-from sqlalchemy.sql import text
+from sqlalchemy import orm
 
 from nova.compute import utils as compute_utils
-from nova import db
-from nova.db.sqlalchemy import api as db_api
-from nova.db.sqlalchemy import api_models
-from nova.db.sqlalchemy import models as main_models
+from nova.db.api import api as api_db_api
+from nova.db.api import models as api_models
 from nova import exception
-from nova.i18n import _, _LW
+from nova.i18n import _
 from nova import objects
 from nova.objects import base
 from nova.objects import fields
@@ -37,11 +32,11 @@ LOG = logging.getLogger(__name__)
 DEPRECATED_FIELDS = ['deleted', 'deleted_at']
 
 
-@db_api.api_context_manager.reader
+@api_db_api.context_manager.reader
 def _aggregate_get_from_db(context, aggregate_id):
     query = context.session.query(api_models.Aggregate).\
-            options(joinedload('_hosts')).\
-            options(joinedload('_metadata'))
+            options(orm.joinedload('_hosts')).\
+            options(orm.joinedload('_metadata'))
     query = query.filter(api_models.Aggregate.id == aggregate_id)
 
     aggregate = query.first()
@@ -52,11 +47,11 @@ def _aggregate_get_from_db(context, aggregate_id):
     return aggregate
 
 
-@db_api.api_context_manager.reader
+@api_db_api.context_manager.reader
 def _aggregate_get_from_db_by_uuid(context, aggregate_uuid):
     query = context.session.query(api_models.Aggregate).\
-            options(joinedload('_hosts')).\
-            options(joinedload('_metadata'))
+            options(orm.joinedload('_hosts')).\
+            options(orm.joinedload('_metadata'))
     query = query.filter(api_models.Aggregate.uuid == aggregate_uuid)
 
     aggregate = query.first()
@@ -69,7 +64,7 @@ def _aggregate_get_from_db_by_uuid(context, aggregate_uuid):
 
 def _host_add_to_db(context, aggregate_id, host):
     try:
-        with db_api.api_context_manager.writer.using(context):
+        with api_db_api.context_manager.writer.using(context):
             # Check to see if the aggregate exists
             _aggregate_get_from_db(context, aggregate_id)
 
@@ -84,7 +79,7 @@ def _host_add_to_db(context, aggregate_id, host):
 
 def _host_delete_from_db(context, aggregate_id, host):
     count = 0
-    with db_api.api_context_manager.writer.using(context):
+    with api_db_api.context_manager.writer.using(context):
         # Check to see if the aggregate exists
         _aggregate_get_from_db(context, aggregate_id)
 
@@ -103,7 +98,7 @@ def _metadata_add_to_db(context, aggregate_id, metadata, max_retries=10,
     all_keys = metadata.keys()
     for attempt in range(max_retries):
         try:
-            with db_api.api_context_manager.writer.using(context):
+            with api_db_api.context_manager.writer.using(context):
                 query = context.session.query(api_models.AggregateMetadata).\
                             filter_by(aggregate_id=aggregate_id)
 
@@ -130,7 +125,7 @@ def _metadata_add_to_db(context, aggregate_id, metadata, max_retries=10,
                                         "aggregate_id": aggregate_id})
                 if new_entries:
                     context.session.execute(
-                        api_models.AggregateMetadata.__table__.insert(),
+                        api_models.AggregateMetadata.__table__.insert(None),
                         new_entries)
 
                 return metadata
@@ -147,7 +142,7 @@ def _metadata_add_to_db(context, aggregate_id, metadata, max_retries=10,
                     LOG.warning(msg)
 
 
-@db_api.api_context_manager.writer
+@api_db_api.context_manager.writer
 def _metadata_delete_from_db(context, aggregate_id, key):
     # Check to see if the aggregate exists
     _aggregate_get_from_db(context, aggregate_id)
@@ -162,7 +157,7 @@ def _metadata_delete_from_db(context, aggregate_id, key):
                             aggregate_id=aggregate_id, metadata_key=key)
 
 
-@db_api.api_context_manager.writer
+@api_db_api.context_manager.writer
 def _aggregate_create_in_db(context, values, metadata=None):
     query = context.session.query(api_models.Aggregate)
     query = query.filter(api_models.Aggregate.name == values['name'])
@@ -186,7 +181,7 @@ def _aggregate_create_in_db(context, values, metadata=None):
     return aggregate
 
 
-@db_api.api_context_manager.writer
+@api_db_api.context_manager.writer
 def _aggregate_delete_from_db(context, aggregate_id):
     # Delete Metadata first
     context.session.query(api_models.AggregateMetadata).\
@@ -201,7 +196,7 @@ def _aggregate_delete_from_db(context, aggregate_id):
         raise exception.AggregateNotFound(aggregate_id=aggregate_id)
 
 
-@db_api.api_context_manager.writer
+@api_db_api.context_manager.writer
 def _aggregate_update_to_db(context, aggregate_id, values):
     aggregate = _aggregate_get_from_db(context, aggregate_id)
 
@@ -248,48 +243,24 @@ class Aggregate(base.NovaPersistentObject, base.NovaObject):
 
     obj_extra_fields = ['availability_zone']
 
-    def __init__(self, *args, **kwargs):
-        super(Aggregate, self).__init__(*args, **kwargs)
-        self._in_api = False
-
     @staticmethod
     def _from_db_object(context, aggregate, db_aggregate):
         for key in aggregate.fields:
             if key == 'metadata':
                 db_key = 'metadetails'
-            elif key == 'uuid':
-                continue
             elif key in DEPRECATED_FIELDS and key not in db_aggregate:
                 continue
             else:
                 db_key = key
             setattr(aggregate, key, db_aggregate[db_key])
 
-        # NOTE(danms): Remove this conditional load (and remove uuid
-        # special cases above) once we're in Newton and have enforced
-        # that all UUIDs in the database are not NULL.
-        if db_aggregate.get('uuid'):
-            aggregate.uuid = db_aggregate['uuid']
-
-        # NOTE: This can be removed when we remove compatibility with
-        # the old aggregate model.
-        if any(f not in db_aggregate for f in DEPRECATED_FIELDS):
-            aggregate.deleted_at = None
-            aggregate.deleted = False
+        # NOTE: This can be removed when we bump Aggregate to v2.0
+        aggregate.deleted_at = None
+        aggregate.deleted = False
 
         aggregate._context = context
         aggregate.obj_reset_changes()
 
-        # NOTE(danms): This needs to come after obj_reset_changes() to make
-        # sure we only save the uuid, if we generate one.
-        # FIXME(danms): Remove this in Newton once we have enforced that
-        # all aggregates have uuids set in the database.
-        if 'uuid' not in aggregate:
-            aggregate.uuid = uuidutils.generate_uuid()
-            LOG.debug('Generating UUID %(uuid)s for aggregate %(agg)i',
-                      dict(uuid=aggregate.uuid, agg=aggregate.id))
-            aggregate.save()
-
         return aggregate
 
     def _assert_no_hosts(self, action):
@@ -298,60 +269,23 @@ class Aggregate(base.NovaPersistentObject, base.NovaObject):
                 action=action,
                 reason='hosts updated inline')
 
-    @property
-    def in_api(self):
-        if self._in_api:
-            return True
-        else:
-            try:
-                _aggregate_get_from_db(self._context, self.id)
-                self._in_api = True
-            except exception.AggregateNotFound:
-                pass
-            return self._in_api
-
     @base.remotable_classmethod
     def get_by_id(cls, context, aggregate_id):
-        try:
-            db_aggregate = _aggregate_get_from_db(context, aggregate_id)
-        except exception.AggregateNotFound:
-            db_aggregate = db.aggregate_get(context, aggregate_id)
+        db_aggregate = _aggregate_get_from_db(context, aggregate_id)
         return cls._from_db_object(context, cls(), db_aggregate)
 
     @base.remotable_classmethod
     def get_by_uuid(cls, context, aggregate_uuid):
-        try:
-            db_aggregate = _aggregate_get_from_db_by_uuid(context,
-                                                          aggregate_uuid)
-        except exception.AggregateNotFound:
-            db_aggregate = db.aggregate_get_by_uuid(context, aggregate_uuid)
+        db_aggregate = _aggregate_get_from_db_by_uuid(context,
+                                                      aggregate_uuid)
         return cls._from_db_object(context, cls(), db_aggregate)
 
-    @staticmethod
-    @db_api.main_context_manager.reader
-    def _ensure_migrated(context):
-        result = context.session.query(main_models.Aggregate).\
-                 filter_by(deleted=0).count()
-        if result:
-            LOG.warning(
-                _LW('Main database contains %(count)i unmigrated aggregates'),
-                {'count': result})
-        return result == 0
-
     @base.remotable
     def create(self):
         if self.obj_attr_is_set('id'):
             raise exception.ObjectActionError(action='create',
                                               reason='already created')
 
-        # NOTE(mdoff): Once we have made it past a point where we know
-        # all aggregates have been migrated, we can remove this. Ideally
-        # in Ocata with a blocker migration to be sure.
-        if not self._ensure_migrated(self._context):
-            raise exception.ObjectActionError(
-                action='create',
-                reason='main database still contains aggregates')
-
         self._assert_no_hosts('create')
         updates = self.obj_get_changes()
         payload = dict(updates)
@@ -360,11 +294,18 @@ class Aggregate(base.NovaPersistentObject, base.NovaObject):
             payload['meta_data'] = payload.pop('metadata')
         if 'uuid' not in updates:
             updates['uuid'] = uuidutils.generate_uuid()
+            self.uuid = updates['uuid']
             LOG.debug('Generated uuid %(uuid)s for aggregate',
                       dict(uuid=updates['uuid']))
         compute_utils.notify_about_aggregate_update(self._context,
                                                     "create.start",
                                                     payload)
+        compute_utils.notify_about_aggregate_action(
+            context=self._context,
+            aggregate=self,
+            action=fields.NotificationAction.CREATE,
+            phase=fields.NotificationPhase.START)
+
         metadata = updates.pop('metadata', None)
         db_aggregate = _aggregate_create_in_db(self._context, updates,
                                                metadata=metadata)
@@ -373,6 +314,11 @@ class Aggregate(base.NovaPersistentObject, base.NovaObject):
         compute_utils.notify_about_aggregate_update(self._context,
                                                     "create.end",
                                                     payload)
+        compute_utils.notify_about_aggregate_action(
+            context=self._context,
+            aggregate=self,
+            action=fields.NotificationAction.CREATE,
+            phase=fields.NotificationPhase.END)
 
     @base.remotable
     def save(self):
@@ -385,37 +331,41 @@ class Aggregate(base.NovaPersistentObject, base.NovaObject):
         compute_utils.notify_about_aggregate_update(self._context,
                                                     "updateprop.start",
                                                     payload)
+        compute_utils.notify_about_aggregate_action(
+            context=self._context,
+            aggregate=self,
+            action=fields.NotificationAction.UPDATE_PROP,
+            phase=fields.NotificationPhase.START)
         updates.pop('id', None)
-        try:
-            db_aggregate = _aggregate_update_to_db(self._context,
-                                                   self.id, updates)
-        except exception.AggregateNotFound:
-            db_aggregate = db.aggregate_update(self._context, self.id, updates)
-
+        db_aggregate = _aggregate_update_to_db(self._context,
+                                               self.id, updates)
         compute_utils.notify_about_aggregate_update(self._context,
                                                     "updateprop.end",
                                                     payload)
+        compute_utils.notify_about_aggregate_action(
+            context=self._context,
+            aggregate=self,
+            action=fields.NotificationAction.UPDATE_PROP,
+            phase=fields.NotificationPhase.END)
         self._from_db_object(self._context, self, db_aggregate)
 
     @base.remotable
     def update_metadata(self, updates):
-        if self.in_api:
-            metadata_delete = _metadata_delete_from_db
-            metadata_add = _metadata_add_to_db
-        else:
-            metadata_delete = db.aggregate_metadata_delete
-            metadata_add = db.aggregate_metadata_add
-
         payload = {'aggregate_id': self.id,
                    'meta_data': updates}
         compute_utils.notify_about_aggregate_update(self._context,
                                                     "updatemetadata.start",
                                                     payload)
+        compute_utils.notify_about_aggregate_action(
+            context=self._context,
+            aggregate=self,
+            action=fields.NotificationAction.UPDATE_METADATA,
+            phase=fields.NotificationPhase.START)
         to_add = {}
         for key, value in updates.items():
             if value is None:
                 try:
-                    metadata_delete(self._context, self.id, key)
+                    _metadata_delete_from_db(self._context, self.id, key)
                 except exception.AggregateMetadataNotFound:
                     pass
                 try:
@@ -425,25 +375,24 @@ class Aggregate(base.NovaPersistentObject, base.NovaObject):
             else:
                 to_add[key] = value
                 self.metadata[key] = value
-        metadata_add(self._context, self.id, to_add)
+        _metadata_add_to_db(self._context, self.id, to_add)
         compute_utils.notify_about_aggregate_update(self._context,
                                                     "updatemetadata.end",
                                                     payload)
+        compute_utils.notify_about_aggregate_action(
+            context=self._context,
+            aggregate=self,
+            action=fields.NotificationAction.UPDATE_METADATA,
+            phase=fields.NotificationPhase.END)
         self.obj_reset_changes(fields=['metadata'])
 
     @base.remotable
     def destroy(self):
-        try:
-            _aggregate_delete_from_db(self._context, self.id)
-        except exception.AggregateNotFound:
-            db.aggregate_delete(self._context, self.id)
+        _aggregate_delete_from_db(self._context, self.id)
 
     @base.remotable
     def add_host(self, host):
-        if self.in_api:
-            _host_add_to_db(self._context, self.id, host)
-        else:
-            db.aggregate_host_add(self._context, self.id, host)
+        _host_add_to_db(self._context, self.id, host)
 
         if self.hosts is None:
             self.hosts = []
@@ -452,10 +401,7 @@ class Aggregate(base.NovaPersistentObject, base.NovaObject):
 
     @base.remotable
     def delete_host(self, host):
-        if self.in_api:
-            _host_delete_from_db(self._context, self.id, host)
-        else:
-            db.aggregate_host_delete(self._context, self.id, host)
+        _host_delete_from_db(self._context, self.id, host)
 
         self.hosts.remove(host)
         self.obj_reset_changes(fields=['hosts'])
@@ -465,20 +411,20 @@ class Aggregate(base.NovaPersistentObject, base.NovaObject):
         return self.metadata.get('availability_zone', None)
 
 
-@db_api.api_context_manager.reader
+@api_db_api.context_manager.reader
 def _get_all_from_db(context):
     query = context.session.query(api_models.Aggregate).\
-            options(joinedload('_hosts')).\
-            options(joinedload('_metadata'))
+            options(orm.joinedload('_hosts')).\
+            options(orm.joinedload('_metadata'))
 
     return query.all()
 
 
-@db_api.api_context_manager.reader
+@api_db_api.context_manager.reader
 def _get_by_host_from_db(context, host, key=None):
     query = context.session.query(api_models.Aggregate).\
-            options(joinedload('_hosts')).\
-            options(joinedload('_metadata'))
+            options(orm.joinedload('_hosts')).\
+            options(orm.joinedload('_metadata'))
     query = query.join('_hosts')
     query = query.filter(api_models.AggregateHost.host == host)
 
@@ -489,13 +435,49 @@ def _get_by_host_from_db(context, host, key=None):
     return query.all()
 
 
-@db_api.api_context_manager.reader
-def _get_by_metadata_key_from_db(context, key):
+@api_db_api.context_manager.reader
+def _get_by_metadata_from_db(context, key=None, value=None):
+    assert(key is not None or value is not None)
     query = context.session.query(api_models.Aggregate)
     query = query.join("_metadata")
-    query = query.filter(api_models.AggregateMetadata.key == key)
-    query = query.options(contains_eager("_metadata"))
-    query = query.options(joinedload("_hosts"))
+    if key is not None:
+        query = query.filter(api_models.AggregateMetadata.key == key)
+    if value is not None:
+        query = query.filter(api_models.AggregateMetadata.value == value)
+    query = query.options(orm.contains_eager("_metadata"))
+    query = query.options(orm.joinedload("_hosts"))
+
+    return query.all()
+
+
+@api_db_api.context_manager.reader
+def _get_non_matching_by_metadata_keys_from_db(context, ignored_keys,
+                                               key_prefix, value):
+    """Filter aggregates based on non matching metadata.
+
+    Find aggregates with at least one ${key_prefix}*[=${value}] metadata where
+    the metadata key are not in the ignored_keys list.
+
+    :return: Aggregates with any metadata entry:
+        - whose key starts with `key_prefix`; and
+        - whose value is `value` and
+        - whose key is *not* in the `ignored_keys` list.
+    """
+
+    if not key_prefix:
+        raise ValueError(_('key_prefix mandatory field.'))
+
+    query = context.session.query(api_models.Aggregate)
+    query = query.join("_metadata")
+    query = query.filter(api_models.AggregateMetadata.value == value)
+    query = query.filter(api_models.AggregateMetadata.key.like(
+        key_prefix + '%'))
+    if len(ignored_keys) > 0:
+        query = query.filter(~api_models.AggregateMetadata.key.in_(
+            ignored_keys))
+
+    query = query.options(orm.contains_eager("_metadata"))
+    query = query.options(orm.joinedload("_hosts"))
 
     return query.all()
 
@@ -506,20 +488,13 @@ class AggregateList(base.ObjectListBase, base.NovaObject):
     # Version 1.1: Added key argument to get_by_host()
     #              Aggregate <= version 1.1
     # Version 1.2: Added get_by_metadata_key
-    VERSION = '1.2'
+    # Version 1.3: Added get_by_metadata
+    VERSION = '1.3'
 
     fields = {
         'objects': fields.ListOfObjectsField('Aggregate'),
         }
 
-    # NOTE(mdoff): Calls to this can be removed when we remove
-    # compatibility with the old aggregate model.
-    @staticmethod
-    def _fill_deprecated(db_aggregate):
-        db_aggregate['deleted_at'] = None
-        db_aggregate['deleted'] = False
-        return db_aggregate
-
     @classmethod
     def _filter_db_aggregates(cls, db_aggregates, hosts):
         if not isinstance(hosts, set):
@@ -534,89 +509,70 @@ class AggregateList(base.ObjectListBase, base.NovaObject):
 
     @base.remotable_classmethod
     def get_all(cls, context):
-        api_db_aggregates = [cls._fill_deprecated(agg) for agg in
-                                _get_all_from_db(context)]
-        db_aggregates = db.aggregate_get_all(context)
+        db_aggregates = _get_all_from_db(context)
         return base.obj_make_list(context, cls(context), objects.Aggregate,
-                                  db_aggregates + api_db_aggregates)
+                                  db_aggregates)
 
     @base.remotable_classmethod
     def get_by_host(cls, context, host, key=None):
-        api_db_aggregates = [cls._fill_deprecated(agg) for agg in
-                            _get_by_host_from_db(context, host, key=key)]
-        db_aggregates = db.aggregate_get_by_host(context, host, key=key)
+        db_aggregates = _get_by_host_from_db(context, host, key=key)
         return base.obj_make_list(context, cls(context), objects.Aggregate,
-                                  db_aggregates + api_db_aggregates)
+                                  db_aggregates)
 
     @base.remotable_classmethod
     def get_by_metadata_key(cls, context, key, hosts=None):
-        api_db_aggregates = [cls._fill_deprecated(agg) for agg in
-                            _get_by_metadata_key_from_db(context, key=key)]
-        db_aggregates = db.aggregate_get_by_metadata_key(context, key=key)
-
-        all_aggregates = db_aggregates + api_db_aggregates
+        db_aggregates = _get_by_metadata_from_db(context, key=key)
         if hosts is not None:
-            all_aggregates = cls._filter_db_aggregates(all_aggregates, hosts)
+            db_aggregates = cls._filter_db_aggregates(db_aggregates, hosts)
         return base.obj_make_list(context, cls(context), objects.Aggregate,
-                                  all_aggregates)
+                                  db_aggregates)
 
+    @base.remotable_classmethod
+    def get_by_metadata(cls, context, key=None, value=None):
+        """Return aggregates with a metadata key set to value.
 
-@db_api.main_context_manager.reader
-def _get_main_db_aggregate_ids(context, limit):
-    from nova.db.sqlalchemy import models
-    return [x[0] for x in context.session.query(models.Aggregate.id).
-            filter_by(deleted=0).
-            limit(limit)]
+        This returns a list of all aggregates that have a metadata key
+        set to some value. If key is specified, then only values for
+        that key will qualify.
+        """
+        db_aggregates = _get_by_metadata_from_db(context, key=key, value=value)
+        return base.obj_make_list(context, cls(context), objects.Aggregate,
+                                  db_aggregates)
 
+    @classmethod
+    def get_non_matching_by_metadata_keys(cls, context, ignored_keys,
+                                          key_prefix, value):
+        """Return aggregates that are not matching with metadata.
 
-def migrate_aggregates(ctxt, count):
-    main_db_ids = _get_main_db_aggregate_ids(ctxt, count)
-    if not main_db_ids:
-        return 0, 0
+        For example, we have aggregates with metadata as below:
 
-    count_all = len(main_db_ids)
-    count_hit = 0
+            'agg1' with trait:HW_CPU_X86_MMX="required"
+            'agg2' with trait:HW_CPU_X86_SGX="required"
+            'agg3' with trait:HW_CPU_X86_MMX="required"
+            'agg3' with trait:HW_CPU_X86_SGX="required"
 
-    for aggregate_id in main_db_ids:
-        try:
-            aggregate = Aggregate.get_by_id(ctxt, aggregate_id)
-            remove = ['metadata', 'hosts']
-            values = {field: getattr(aggregate, field)
-                      for field in aggregate.fields if field not in remove}
-            _aggregate_create_in_db(ctxt, values, metadata=aggregate.metadata)
-            for host in aggregate.hosts:
-                _host_add_to_db(ctxt, aggregate_id, host)
-            count_hit += 1
-            db.aggregate_delete(ctxt, aggregate.id)
-        except exception.AggregateNotFound:
-            LOG.warning(
-                _LW('Aggregate id %(id)i disappeared during migration'),
-                {'id': aggregate_id})
-        except (exception.AggregateNameExists) as e:
-            LOG.error(str(e))
+        Assume below request:
 
-    return count_all, count_hit
+            aggregate_obj.AggregateList.get_non_matching_by_metadata_keys(
+                self.context,
+                ['trait:HW_CPU_X86_MMX'],
+                'trait:',
+                value='required')
 
+        It will return 'agg2' and 'agg3' as aggregates that are not matching
+        with metadata.
 
-def _adjust_autoincrement(context, value):
-    engine = db_api.get_api_engine()
-    if engine.name == 'postgresql':
-        # NOTE(danms): If we migrated some aggregates in the above function,
-        # then we will have confused postgres' sequence for the autoincrement
-        # primary key. MySQL does not care about this, but since postgres does,
-        # we need to reset this to avoid a failure on the next aggregate
-        # creation.
-        engine.execute(
-            text('ALTER SEQUENCE aggregates_id_seq RESTART WITH %i;' % (
-                value)))
+        :param context: The security context
+        :param ignored_keys: List of keys to match with the aggregate metadata
+                     keys that starts with key_prefix.
+        :param key_prefix: Only compares metadata keys that starts with the
+                           key_prefix
+        :param value: Value of metadata
 
-
-@db_api.api_context_manager.reader
-def _get_max_aggregate_id(context):
-    return context.session.query(func.max(api_models.Aggregate.id)).one()[0]
-
-
-def migrate_aggregate_reset_autoincrement(ctxt, count):
-    max_id = _get_max_aggregate_id(ctxt) or 0
-    _adjust_autoincrement(ctxt, max_id + 1)
-    return 0, 0
+        :returns: List of aggregates that doesn't match metadata keys that
+                  starts with key_prefix with the supplied keys.
+        """
+        db_aggregates = _get_non_matching_by_metadata_keys_from_db(
+            context, ignored_keys, key_prefix, value)
+        return base.obj_make_list(context, objects.AggregateList(context),
+                                  objects.Aggregate, db_aggregates)
