@@ -15,13 +15,15 @@
 
 import copy
 
+from oslo_db import api as oslo_db_api
+from oslo_db.sqlalchemy import update_match
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
 from oslo_utils import versionutils
-import six
 
-from nova.db import api as db
+from nova.db.main import api as db
+from nova.db.main import models as db_models
 from nova import exception
 from nova import objects
 from nova.objects import base
@@ -95,7 +97,8 @@ class PciDevice(base.NovaPersistentObject, base.NovaObject):
     # Version 1.4: Added parent_addr field
     # Version 1.5: Added 2 new device statuses: UNCLAIMABLE and UNAVAILABLE
     # Version 1.6: Added uuid field
-    VERSION = '1.6'
+    # Version 1.7: Added 'vdpa' to 'dev_type' field
+    VERSION = '1.7'
 
     fields = {
         'id': fields.IntegerField(),
@@ -112,7 +115,7 @@ class PciDevice(base.NovaPersistentObject, base.NovaObject):
         'label': fields.StringField(nullable=True),
         'instance_uuid': fields.StringField(nullable=True),
         'request_id': fields.StringField(nullable=True),
-        'extra_info': fields.DictOfStringsField(),
+        'extra_info': fields.DictOfStringsField(default={}),
         'numa_node': fields.IntegerField(nullable=True),
         'parent_addr': fields.StringField(nullable=True),
     }
@@ -137,6 +140,19 @@ class PciDevice(base.NovaPersistentObject, base.NovaObject):
                         status, target_version))
         if target_version < (1, 6) and 'uuid' in primitive:
             del primitive['uuid']
+        if target_version < (1, 7) and 'dev_type' in primitive:
+            dev_type = primitive['dev_type']
+            if dev_type == fields.PciDeviceType.VDPA:
+                raise exception.ObjectActionError(
+                    action='obj_make_compatible',
+                    reason='dev_type=%s not supported in version %s' % (
+                        dev_type, target_version))
+
+    def __repr__(self):
+        return (
+            f'PciDevice(address={self.address}, '
+            f'compute_node_id={self.compute_node_id})'
+        )
 
     def update_device(self, dev_dict):
         """Sync the content from device dictionary to device object.
@@ -165,53 +181,112 @@ class PciDevice(base.NovaPersistentObject, base.NovaObject):
                 # NOTE(ralonsoh): list of parameters currently added to
                 # "extra_info" dict:
                 #     - "capabilities": dict of (strings/list of strings)
+                #     - "parent_ifname": the netdev name of the parent (PF)
+                #        device of a VF
+                #     - "mac_address": the MAC address of the PF
                 extra_info = self.extra_info
-                data = (v if isinstance(v, six.string_types) else
-                        jsonutils.dumps(v))
+                data = v if isinstance(v, str) else jsonutils.dumps(v)
                 extra_info.update({k: data})
                 self.extra_info = extra_info
 
     def __init__(self, *args, **kwargs):
         super(PciDevice, self).__init__(*args, **kwargs)
-        self.obj_reset_changes()
-        self.extra_info = {}
+
         # NOTE(ndipanov): These are required to build an in-memory device tree
         # but don't need to be proper fields (and can't easily be as they would
         # hold circular references)
         self.parent_device = None
         self.child_devices = []
 
+    def obj_load_attr(self, attr):
+        if attr in ['extra_info']:
+            # NOTE(danms): extra_info used to be defaulted during init,
+            # so make sure any bare instantiations of this object can
+            # rely on the expectation that referencing that field will
+            # not fail.
+            self.obj_set_defaults(attr)
+        else:
+            super(PciDevice, self).obj_load_attr(attr)
+
     def __eq__(self, other):
         return compare_pci_device_attributes(self, other)
 
     def __ne__(self, other):
         return not (self == other)
 
-    @staticmethod
-    def _from_db_object(context, pci_device, db_dev):
+    @classmethod
+    def populate_dev_uuids(cls, context, count):
+        @db.pick_context_manager_reader
+        def get_devs_no_uuid(context):
+            return context.session.query(db_models.PciDevice).\
+                    filter_by(uuid=None).limit(count).all()
+
+        db_devs = get_devs_no_uuid(context)
+
+        done = 0
+        for db_dev in db_devs:
+            cls._create_uuid(context, db_dev['id'])
+            done += 1
+
+        return done, done
+
+    @classmethod
+    def _from_db_object(cls, context, pci_device, db_dev):
         for key in pci_device.fields:
             if key == 'uuid' and db_dev['uuid'] is None:
-                # Older records might not have a uuid field set in the
-                # database so we need to skip those here and auto-generate
-                # a uuid later below.
-                continue
-            elif key != 'extra_info':
-                setattr(pci_device, key, db_dev[key])
-            else:
-                extra_info = db_dev.get("extra_info")
+                # NOTE(danms): While the records could be nullable,
+                # generate a UUID on read since the object requires it
+                dev_id = db_dev['id']
+                db_dev[key] = cls._create_uuid(context, dev_id)
+
+            if key == 'extra_info':
+                extra_info = db_dev.get('extra_info')
                 pci_device.extra_info = jsonutils.loads(extra_info)
+                continue
+
+            setattr(pci_device, key, db_dev[key])
+
         pci_device._context = context
         pci_device.obj_reset_changes()
-
-        # TODO(jaypipes): Remove in 2.0 version of object. This does an inline
-        # migration to populate the uuid field. A similar inline migration is
-        # performed in the save() method.
-        if db_dev['uuid'] is None:
-            pci_device.uuid = uuidutils.generate_uuid()
-            pci_device.save()
-
         return pci_device
 
+    @staticmethod
+    @oslo_db_api.wrap_db_retry(max_retries=1, retry_on_deadlock=True)
+    def _create_uuid(context, dev_id):
+        # NOTE(mdbooth): This method is only required until uuid is made
+        # non-nullable in a future release.
+
+        # NOTE(mdbooth): We wrap this method in a retry loop because it can
+        # fail (safely) on multi-master galera if concurrent updates happen on
+        # different masters. It will never fail on single-master. We can only
+        # ever need one retry.
+
+        uuid = uuidutils.generate_uuid()
+        values = {'uuid': uuid}
+        compare = db_models.PciDevice(id=dev_id, uuid=None)
+
+        # NOTE(mdbooth): We explicitly use an independent transaction context
+        # here so as not to fail if:
+        # 1. We retry.
+        # 2. We're in a read transaction. This is an edge case of what's
+        #    normally a read operation. Forcing everything (transitively) which
+        #    reads a PCI device to be in a write transaction for a narrow
+        #    temporary edge case is undesirable.
+        tctxt = db.get_context_manager(context).writer.independent
+        with tctxt.using(context):
+            query = context.session.query(db_models.PciDevice).\
+                        filter_by(id=dev_id)
+
+            try:
+                query.update_on_match(compare, 'id', values)
+            except update_match.NoRowsMatched:
+                # We can only get here if we raced, and another writer already
+                # gave this PCI device a UUID
+                result = query.one()
+                uuid = result['uuid']
+
+        return uuid
+
     @base.remotable_classmethod
     def get_by_dev_addr(cls, context, compute_node_id, dev_addr):
         db_dev = db.pci_device_get_by_addr(
@@ -231,6 +306,10 @@ class PciDevice(base.NovaPersistentObject, base.NovaObject):
         thus we should not reset changes here for fields from dict.
         """
         pci_device = cls()
+        # NOTE(danms): extra_info used to always be defaulted during init,
+        # so make sure we replicate that behavior outside of init here
+        # for compatibility reasons.
+        pci_device.obj_set_defaults('extra_info')
         pci_device.update_device(dev_dict)
         pci_device.status = fields.PciDeviceStatus.AVAILABLE
         pci_device.uuid = uuidutils.generate_uuid()
@@ -283,7 +362,9 @@ class PciDevice(base.NovaPersistentObject, base.NovaObject):
             self._bulk_update_status(vfs_list,
                                            fields.PciDeviceStatus.UNCLAIMABLE)
 
-        elif self.dev_type == fields.PciDeviceType.SRIOV_VF:
+        elif self.dev_type in (
+            fields.PciDeviceType.SRIOV_VF, fields.PciDeviceType.VDPA
+        ):
             # Update VF status to CLAIMED if it's parent has not been
             # previously allocated or claimed
             # When claiming/allocating a VF, it's parent PF becomes
@@ -343,7 +424,9 @@ class PciDevice(base.NovaPersistentObject, base.NovaObject):
             self._bulk_update_status(vfs_list,
                                      fields.PciDeviceStatus.UNAVAILABLE)
 
-        elif (self.dev_type == fields.PciDeviceType.SRIOV_VF):
+        elif self.dev_type in (
+            fields.PciDeviceType.SRIOV_VF, fields.PciDeviceType.VDPA
+        ):
             parent = self.parent_device
             if parent:
                 if parent.status not in parent_ok_statuses:
@@ -402,7 +485,9 @@ class PciDevice(base.NovaPersistentObject, base.NovaObject):
             self._bulk_update_status(vfs_list,
                                      fields.PciDeviceStatus.AVAILABLE)
             free_devs.extend(vfs_list)
-        if self.dev_type == fields.PciDeviceType.SRIOV_VF:
+        if self.dev_type in (
+            fields.PciDeviceType.SRIOV_VF, fields.PciDeviceType.VDPA
+        ):
             # Set PF status to AVAILABLE if all of it's VFs are free
             parent = self.parent_device
             if not parent:
@@ -435,6 +520,13 @@ class PciDevice(base.NovaPersistentObject, base.NovaObject):
     def is_available(self):
         return self.status == fields.PciDeviceStatus.AVAILABLE
 
+    @property
+    def mac_address(self):
+        """The MAC address of the PF physical device or None if the device is
+        not a PF or if the MAC is not available.
+        """
+        return self.extra_info.get('mac_address')
+
 
 @base.NovaObjectRegistry.register
 class PciDeviceList(base.ObjectListBase, base.NovaObject):
@@ -474,3 +566,6 @@ class PciDeviceList(base.ObjectListBase, base.NovaObject):
                                                            parent_addr)
         return base.obj_make_list(context, cls(context), objects.PciDevice,
                                   db_dev_list)
+
+    def __repr__(self):
+        return f"PciDeviceList(objects={[repr(obj) for obj in self.objects]})"
