@@ -24,6 +24,7 @@ inline callbacks.
 import nova.monkey_patch  # noqa
 
 import abc
+import builtins
 import collections
 import contextlib
 import copy
@@ -49,23 +50,19 @@ from oslo_utils import timeutils
 from oslo_versionedobjects import fixture as ovo_fixture
 from oslotest import base
 from oslotest import mock_fixture
-import six
-from six.moves import builtins
 from sqlalchemy.dialects import sqlite
 import testtools
 
 from nova.api.openstack import wsgi_app
 from nova.compute import rpcapi as compute_rpcapi
 from nova import context
-from nova.db.sqlalchemy import api as sqlalchemy_api
+from nova.db.main import api as db_api
 from nova import exception
 from nova import objects
 from nova.objects import base as objects_base
 from nova import quota
 from nova.tests import fixtures as nova_fixtures
-from nova.tests.unit import conf_fixture
 from nova.tests.unit import matchers
-from nova.tests.unit import policy_fixture
 from nova import utils
 from nova.virt import images
 
@@ -180,6 +177,9 @@ class TestCase(base.BaseTestCase):
         with fixtures.EnvironmentVariable('OS_LOG_CAPTURE', '0'):
             super(TestCase, self).setUp()
 
+        self.useFixture(
+            nova_fixtures.PropagateTestCaseIdToChildEventlets(self.id()))
+
         # How many of which service we've started. {$service-name: $count}
         self._service_fixture_count = collections.defaultdict(int)
 
@@ -207,7 +207,7 @@ class TestCase(base.BaseTestCase):
             self.fixture.config(lock_path=lock_path,
                                 group='oslo_concurrency')
 
-        self.useFixture(conf_fixture.ConfFixture(CONF))
+        self.useFixture(nova_fixtures.ConfFixture(CONF))
 
         if self.STUB_RPC:
             self.useFixture(nova_fixtures.RPCFixture('nova.test'))
@@ -265,7 +265,7 @@ class TestCase(base.BaseTestCase):
 
         self.addCleanup(self._clear_attrs)
         self.useFixture(fixtures.EnvironmentVariable('http_proxy'))
-        self.policy = self.useFixture(policy_fixture.PolicyFixture())
+        self.policy = self.useFixture(nova_fixtures.PolicyFixture())
 
         self.useFixture(nova_fixtures.PoisonFunctions())
 
@@ -355,7 +355,7 @@ class TestCase(base.BaseTestCase):
         self.useFixture(fixtures.MonkeyPatch(old, new))
 
     @staticmethod
-    def patch_exists(patched_path, result):
+    def patch_exists(patched_path, result, other=None):
         """Provide a static method version of patch_exists(), which if you
         haven't already imported nova.test can be slightly easier to
         use as a context manager within a test method via:
@@ -364,7 +364,7 @@ class TestCase(base.BaseTestCase):
                 with self.patch_exists(path, True):
                     ...
         """
-        return patch_exists(patched_path, result)
+        return patch_exists(patched_path, result, other)
 
     @staticmethod
     def patch_open(patched_path, read_data):
@@ -385,18 +385,9 @@ class TestCase(base.BaseTestCase):
 
     def enforce_fk_constraints(self, engine=None):
         if engine is None:
-            engine = sqlalchemy_api.get_engine()
+            engine = db_api.get_engine()
         dialect = engine.url.get_dialect()
         if dialect == sqlite.dialect:
-            # We're seeing issues with foreign key support in SQLite 3.6.20
-            # SQLAlchemy doesn't support it at all with < SQLite 3.6.19
-            # It works fine in SQLite 3.7.
-            # So return early to skip this test if running SQLite < 3.7
-            import sqlite3
-            tup = sqlite3.sqlite_version_info
-            if tup[0] < 3 or (tup[0] == 3 and tup[1] < 7):
-                self.skipTest(
-                    'sqlite version too old for reliable SQLA foreign_keys')
             engine.connect().execute("PRAGMA foreign_keys = ON")
 
     def start_service(self, name, host=None, cell_name=None, **kwargs):
@@ -540,9 +531,9 @@ class TestCase(base.BaseTestCase):
         matcher.
 
         """
-        if isinstance(expected, six.string_types):
+        if isinstance(expected, str):
             expected = jsonutils.loads(expected)
-        if isinstance(observed, six.string_types):
+        if isinstance(observed, str):
             observed = jsonutils.loads(observed)
 
         def sort_key(x):
@@ -572,7 +563,7 @@ class TestCase(base.BaseTestCase):
                 self.assertEqual(
                     expected_keys, observed_keys,
                     'path: %s. Dict keys are not equal' % path)
-                for key in list(six.iterkeys(expected)):
+                for key in expected:
                     inner(expected[key], observed[key], path + '.%s' % key)
             elif (isinstance(expected, (list, tuple, set)) and
                       isinstance(observed, (list, tuple, set))):
@@ -645,8 +636,8 @@ class TestCase(base.BaseTestCase):
 
         for name in sorted(implmethods.keys()):
             # NOTE(stephenfin): We ignore type annotations
-            baseargs = utils.getargspec(basemethods[name])[:-1]
-            implargs = utils.getargspec(implmethods[name])[:-1]
+            baseargs = inspect.getfullargspec(basemethods[name])[:-1]
+            implargs = inspect.getfullargspec(implmethods[name])[:-1]
 
             self.assertEqual(baseargs, implargs,
                              "%s args don't match base class %s" %
@@ -719,7 +710,7 @@ class SubclassSignatureTestCase(testtools.TestCase, metaclass=abc.ABCMeta):
                 # instead.
                 method = getattr(method, '__wrapped__')
 
-            argspecs[name] = utils.getargspec(method)
+            argspecs[name] = inspect.getfullargspec(method)
 
         return argspecs
 
@@ -755,9 +746,12 @@ class SubclassSignatureTestCase(testtools.TestCase, metaclass=abc.ABCMeta):
 class TimeOverride(fixtures.Fixture):
     """Fixture to start and remove time override."""
 
+    def __init__(self, override_time=None):
+        self.override_time = override_time
+
     def setUp(self):
         super(TimeOverride, self).setUp()
-        timeutils.set_time_override()
+        timeutils.set_time_override(override_time=self.override_time)
         self.addCleanup(timeutils.clear_time_override)
 
 
@@ -851,10 +845,12 @@ class ContainKeyValue(object):
 
 
 @contextlib.contextmanager
-def patch_exists(patched_path, result):
+def patch_exists(patched_path, result, other=None):
     """Selectively patch os.path.exists() so that if it's called with
     patched_path, return result.  Calls with any other path are passed
-    through to the real os.path.exists() function.
+    through to the real os.path.exists() function if other is not provided.
+    If other is provided then that will be the result of the call on paths
+    other than patched_path.
 
     Either import and use as a decorator / context manager, or use the
     nova.TestCase.patch_exists() static method as a context manager.
@@ -888,7 +884,10 @@ def patch_exists(patched_path, result):
     def fake_exists(path):
         if path == patched_path:
             return result
-        return real_exists(path)
+        elif other is not None:
+            return other
+        else:
+            return real_exists(path)
 
     with mock.patch.object(os.path, "exists") as mock_exists:
         mock_exists.side_effect = fake_exists
@@ -926,7 +925,7 @@ def patch_open(patched_path, read_data):
     selective patching based on the path.  In this case something like
     like this may be more appropriate:
 
-        @mock.patch(six.moves.builtins, 'open')
+        @mock.patch('builtins.open')
         def test_my_code(self, mock_open):
             ...
             mock_open.assert_called_once_with(path)
@@ -939,6 +938,6 @@ def patch_open(patched_path, read_data):
             return m(patched_path)
         return real_open(path, *args, **kwargs)
 
-    with mock.patch.object(builtins, 'open') as mock_open:
+    with mock.patch('builtins.open') as mock_open:
         mock_open.side_effect = selective_fake_open
         yield m
