@@ -20,12 +20,11 @@ from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
 from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import uuidutils
-import six
 import webob
 
 from nova.api.openstack.compute import security_groups as secgroups_v21
 from nova import context as context_maker
-import nova.db.api
+import nova.db.main.api
 from nova import exception
 from nova.network import model
 from nova.network import neutron as neutron_api
@@ -121,11 +120,11 @@ class MockClient(object):
 
     def create_security_group(self, body=None):
         s = body.get('security_group')
-        if not isinstance(s.get('name', ''), six.string_types):
+        if not isinstance(s.get('name', ''), str):
             msg = ('BadRequest: Invalid input for name. Reason: '
                    'None is not a valid string.')
             raise n_exc.BadRequest(message=msg)
-        if not isinstance(s.get('description.', ''), six.string_types):
+        if not isinstance(s.get('description.', ''), str):
             msg = ('BadRequest: Invalid input for description. Reason: '
                    'None is not a valid string.')
             raise n_exc.BadRequest(message=msg)
@@ -637,7 +636,7 @@ class TestSecurityGroupsV21(test.TestCase):
             _context, instance_obj.Instance(), db_inst,
             expected_attrs=instance_obj.INSTANCE_DEFAULT_FIELDS)
         neutron = neutron_api.API()
-        with mock.patch.object(nova.db.api, 'instance_get_by_uuid',
+        with mock.patch.object(nova.db.main.api, 'instance_get_by_uuid',
                                return_value=db_inst):
             neutron.allocate_for_instance(_context, instance, None,
                                           security_groups=[sg['id']])
