@@ -36,7 +36,6 @@ from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
 from oslo_utils.fixture import uuidsentinel as uuids
 import requests
-import six
 import webob
 
 from nova.api.metadata import base
@@ -359,8 +358,9 @@ class MetadataTestCase(test.TestCase):
                      'delete_on_termination': None,
                      'device_name': '/dev/sdb'})]
 
-        self.stub_out('nova.db.api.block_device_mapping_get_all_by_instance',
-                      fake_bdm_get)
+        self.stub_out(
+            'nova.db.main.api.block_device_mapping_get_all_by_instance',
+            fake_bdm_get)
 
         expected = {'ami': 'sda1',
                     'root': '/dev/sda1',
@@ -933,7 +933,7 @@ class OpenStackMetadataTestCase(test.TestCase):
                 # Verify the warning message is the one we expect which is the
                 # first and only arg to the first and only call to the warning.
                 self.assertIn('Passing insecure dynamic vendordata requests',
-                              six.text_type(warning_calls[0][0]))
+                              str(warning_calls[0][0]))
             self.assertEqual('10.0.0.1', vd['static'].get('ldap'))
             self.assertEqual('10.0.0.2', vd['static'].get('ad'))
 
@@ -1462,20 +1462,17 @@ class MetadataHandlerTestCase(test.TestCase):
                        for c in range(ord('a'), ord('z'))]
         mock_client.list_subnets.return_value = {
             'subnets': subnet_list}
+        mock_client.list_ports.side_effect = fake_list_ports
 
-        with mock.patch.object(
-                mock_client, 'list_ports',
-                side_effect=fake_list_ports) as mock_list_ports:
+        response = fake_request(
+            self, self.mdinst,
+            relpath="/2009-04-04/user-data",
+            address="192.192.192.2",
+            fake_get_metadata_by_instance_id=self._fake_x_get_metadata,
+            headers={'X-Forwarded-For': '192.192.192.2',
+                     'X-Metadata-Provider': proxy_lb_id})
 
-            response = fake_request(
-                self, self.mdinst,
-                relpath="/2009-04-04/user-data",
-                address="192.192.192.2",
-                fake_get_metadata_by_instance_id=self._fake_x_get_metadata,
-                headers={'X-Forwarded-For': '192.192.192.2',
-                         'X-Metadata-Provider': proxy_lb_id})
-
-            self.assertEqual(3, mock_list_ports.call_count)
+        self.assertEqual(3, mock_client.list_ports.call_count)
 
         self.assertEqual(200, response.status_int)
 
