@@ -17,12 +17,11 @@
 import ddt
 import mock
 from oslo_db.sqlalchemy import enginefacade
-from six.moves import range
 
 from nova.compute import api as compute
 import nova.conf
 from nova import context
-from nova.db.sqlalchemy import models as sqa_models
+from nova.db.main import models
 from nova import exception
 from nova import objects
 from nova import quota
@@ -65,7 +64,7 @@ class QuotaIntegrationTestCase(test.TestCase):
         self.context = context.RequestContext(self.user_id,
                                               self.project_id,
                                               is_admin=True)
-        self.inst_type = objects.Flavor.get_by_name(self.context, 'm1.small')
+        self.flavor = objects.Flavor.get_by_name(self.context, 'm1.small')
 
         self.useFixture(nova_fixtures.GlanceFixture(self))
 
@@ -107,9 +106,9 @@ class QuotaIntegrationTestCase(test.TestCase):
             self._create_instance()
         image_uuid = 'cedef40a-ed67-4d10-800e-17455edce175'
         try:
-            self.compute_api.create(self.context, min_count=1, max_count=1,
-                                    instance_type=self.inst_type,
-                                    image_href=image_uuid)
+            self.compute_api.create(
+                self.context, min_count=1, max_count=1,
+                flavor=self.flavor, image_href=image_uuid)
         except exception.QuotaError as e:
             expected_kwargs = {'code': 413,
                                'req': '1, 1',
@@ -124,9 +123,9 @@ class QuotaIntegrationTestCase(test.TestCase):
         self._create_instance()
         image_uuid = 'cedef40a-ed67-4d10-800e-17455edce175'
         try:
-            self.compute_api.create(self.context, min_count=1, max_count=1,
-                                    instance_type=self.inst_type,
-                                    image_href=image_uuid)
+            self.compute_api.create(
+                self.context, min_count=1, max_count=1, flavor=self.flavor,
+                image_href=image_uuid)
         except exception.QuotaError as e:
             expected_kwargs = {'code': 413,
                                'req': '1',
@@ -150,27 +149,22 @@ class QuotaIntegrationTestCase(test.TestCase):
         for i in range(CONF.quota.metadata_items + 1):
             metadata['key%s' % i] = 'value%s' % i
         image_uuid = 'cedef40a-ed67-4d10-800e-17455edce175'
-        self.assertRaises(exception.QuotaError, self.compute_api.create,
-                                            self.context,
-                                            min_count=1,
-                                            max_count=1,
-                                            instance_type=self.inst_type,
-                                            image_href=image_uuid,
-                                            metadata=metadata)
+        self.assertRaises(
+            exception.QuotaError, self.compute_api.create,
+            self.context, min_count=1, max_count=1, flavor=self.flavor,
+            image_href=image_uuid, metadata=metadata)
 
     def _create_with_injected_files(self, files):
         api = self.compute_api
         image_uuid = 'cedef40a-ed67-4d10-800e-17455edce175'
-        api.create(self.context, min_count=1, max_count=1,
-                instance_type=self.inst_type, image_href=image_uuid,
-                injected_files=files)
+        api.create(
+            self.context, min_count=1, max_count=1, flavor=self.flavor,
+            image_href=image_uuid, injected_files=files)
 
     def test_no_injected_files(self):
         api = self.compute_api
         image_uuid = 'cedef40a-ed67-4d10-800e-17455edce175'
-        api.create(self.context,
-                   instance_type=self.inst_type,
-                   image_href=image_uuid)
+        api.create(self.context, flavor=self.flavor, image_href=image_uuid)
 
     def test_max_injected_files(self):
         files = []
@@ -602,9 +596,9 @@ class DbQuotaDriverTestCase(test.TestCase):
                 'injected_file_path_bytes': 127,
                 }
 
-        self.stub_out('nova.db.api.quota_get_all_by_project_and_user',
+        self.stub_out('nova.db.main.api.quota_get_all_by_project_and_user',
                        fake_qgabpau)
-        self.stub_out('nova.db.api.quota_get_all_by_project', fake_qgabp)
+        self.stub_out('nova.db.main.api.quota_get_all_by_project', fake_qgabp)
 
         self._stub_quota_class_get_all_by_name()
 
@@ -757,7 +751,7 @@ class DbQuotaDriverTestCase(test.TestCase):
             return dict(
                 test_resource=dict(in_use=20),
                 )
-        self.stub_out('nova.db.api.quota_get', fake_quota_get)
+        self.stub_out('nova.db.main.api.quota_get', fake_quota_get)
 
     def _stub_get_by_project(self):
         def fake_qgabp(context, project_id):
@@ -772,13 +766,13 @@ class DbQuotaDriverTestCase(test.TestCase):
         def fake_quota_get_all(context, project_id):
             self.calls.append('quota_get_all')
             self.assertEqual(project_id, 'test_project')
-            return [sqa_models.ProjectUserQuota(resource='instances',
-                                                hard_limit=5),
-                    sqa_models.ProjectUserQuota(resource='cores',
-                                                hard_limit=2)]
+            return [
+                models.ProjectUserQuota(resource='instances', hard_limit=5),
+                models.ProjectUserQuota(resource='cores', hard_limit=2),
+            ]
 
-        self.stub_out('nova.db.api.quota_get_all_by_project', fake_qgabp)
-        self.stub_out('nova.db.api.quota_get_all', fake_quota_get_all)
+        self.stub_out('nova.db.main.api.quota_get_all_by_project', fake_qgabp)
+        self.stub_out('nova.db.main.api.quota_get_all', fake_quota_get_all)
 
         self._stub_quota_class_get_all_by_name()
         self._stub_quota_class_get_default()
@@ -1416,13 +1410,13 @@ class DbQuotaDriverTestCase(test.TestCase):
             self.calls.append('quota_get_all_by_project_and_user')
             return {'instances': 2, 'cores': -1}
 
-        self.stub_out('nova.db.api.quota_get_all_by_project',
+        self.stub_out('nova.db.main.api.quota_get_all_by_project',
                        fake_quota_get_all_by_project)
         self.stub_out('nova.quota.DbQuotaDriver.get_project_quotas',
                        fake_get_project_quotas)
         self.stub_out('nova.quota.DbQuotaDriver._process_quotas',
                        fake_process_quotas_in_get_user_quotas)
-        self.stub_out('nova.db.api.quota_get_all_by_project_and_user',
+        self.stub_out('nova.db.main.api.quota_get_all_by_project_and_user',
                        fake_qgabpau)
 
     def test_get_settable_quotas_with_user(self):
