@@ -13,7 +13,6 @@
 #    under the License.
 
 import datetime
-import hashlib
 import os
 import os.path
 import tempfile
@@ -31,7 +30,7 @@ from oslo_context import context as common_context
 from oslo_context import fixture as context_fixture
 from oslo_utils import encodeutils
 from oslo_utils import fixture as utils_fixture
-import six
+from oslo_utils.secretutils import md5
 
 from nova import context
 from nova import exception
@@ -74,24 +73,25 @@ class GenericUtilsTestCase(test.NoDBTestCase):
         self.assertEqual(('', ''), result)
 
     def test_hostname_unicode_sanitization(self):
-        hostname = u"\u7684.test.example.com"
-        self.assertEqual("test.example.com",
-                         utils.sanitize_hostname(hostname))
+        hostname = u'\u7684myamazinghostname'
+        self.assertEqual(
+            'myamazinghostname', utils.sanitize_hostname(hostname))
 
     def test_hostname_sanitize_periods(self):
-        hostname = "....test.example.com..."
-        self.assertEqual("test.example.com",
-                         utils.sanitize_hostname(hostname))
+        hostname = '....test.example.com...'
+        self.assertEqual(
+            'test-example-com', utils.sanitize_hostname(hostname))
 
     def test_hostname_sanitize_dashes(self):
-        hostname = "----test.example.com---"
-        self.assertEqual("test.example.com",
-                         utils.sanitize_hostname(hostname))
+        hostname = '----my-amazing-hostname---'
+        self.assertEqual(
+            "my-amazing-hostname", utils.sanitize_hostname(hostname))
 
     def test_hostname_sanitize_characters(self):
         hostname = "(#@&$!(@*--#&91)(__=+--test-host.example!!.com-0+"
-        self.assertEqual("91----test-host.example.com-0",
-                         utils.sanitize_hostname(hostname))
+        self.assertEqual(
+            "91----test-host-example-com-0",
+            utils.sanitize_hostname(hostname))
 
     def test_hostname_translate(self):
         hostname = "<}\x1fh\x10e\x08l\x02l\x05o\x12!{>"
@@ -100,20 +100,20 @@ class GenericUtilsTestCase(test.NoDBTestCase):
     def test_hostname_has_default(self):
         hostname = u"\u7684hello"
         defaultname = "Server-1"
-        self.assertEqual("hello", utils.sanitize_hostname(hostname,
-                                                          defaultname))
+        self.assertEqual(
+            "hello", utils.sanitize_hostname(hostname, defaultname))
 
     def test_hostname_empty_has_default(self):
         hostname = u"\u7684"
         defaultname = "Server-1"
-        self.assertEqual(defaultname, utils.sanitize_hostname(hostname,
-                                                              defaultname))
+        self.assertEqual(
+            defaultname, utils.sanitize_hostname(hostname, defaultname))
 
     def test_hostname_empty_has_default_too_long(self):
         hostname = u"\u7684"
         defaultname = "a" * 64
-        self.assertEqual("a" * 63, utils.sanitize_hostname(hostname,
-                                                           defaultname))
+        self.assertEqual(
+            "a" * 63, utils.sanitize_hostname(hostname, defaultname))
 
     def test_hostname_empty_no_default(self):
         hostname = u"\u7684"
@@ -204,7 +204,7 @@ class GenericUtilsTestCase(test.NoDBTestCase):
     def test_get_hash_str(self):
         base_str = b"foo"
         base_unicode = u"foo"
-        value = hashlib.md5(base_str).hexdigest()
+        value = md5(base_str, usedforsecurity=False).hexdigest()
         self.assertEqual(
             value, utils.get_hash_str(base_str))
         self.assertEqual(
@@ -215,8 +215,7 @@ class GenericUtilsTestCase(test.NoDBTestCase):
         instance.display_name = u'\u00CD\u00F1st\u00E1\u00F1c\u00E9'
         # should be a bytes string if python2 before conversion
         self.assertIs(str, type(repr(instance)))
-        self.assertIs(six.text_type,
-                      type(utils.get_obj_repr_unicode(instance)))
+        self.assertIs(str, type(utils.get_obj_repr_unicode(instance)))
 
     @mock.patch('oslo_concurrency.processutils.execute')
     def test_ssh_execute(self, mock_execute):
@@ -261,7 +260,7 @@ class TestCachedFile(test.NoDBTestCase):
 
         fake_contents = "lorem ipsum"
 
-        with mock.patch('six.moves.builtins.open',
+        with mock.patch('builtins.open',
                         mock.mock_open(read_data=fake_contents)):
             fresh, data = utils.read_cached_file("/this/is/a/fake")
 
@@ -585,7 +584,7 @@ class ValidateIntegerTestCase(test.NoDBTestCase):
                           max_value=54)
         self.assertRaises(exception.InvalidInput,
                           utils.validate_integer,
-                          six.unichr(129), "UnicodeError",
+                          chr(129), "UnicodeError",
                           max_value=1000)
 
 
@@ -836,7 +835,7 @@ class UT8TestCase(test.NoDBTestCase):
     def test_not_text_type(self):
         return_value = utils.utf8(1)
         self.assertEqual(b"1", return_value)
-        self.assertIsInstance(return_value, six.binary_type)
+        self.assertIsInstance(return_value, bytes)
 
     def test_text_type_with_encoding(self):
         some_value = 'test\u2026config'
@@ -1311,7 +1310,7 @@ class RunOnceTests(test.NoDBTestCase):
         self.dummy_test_func.reset()
         RunOnceTests.fake_logger.reset_mock()
 
-    def test_wrapped_funtions_called_once(self):
+    def test_wrapped_functions_called_once(self):
         self.assertFalse(self.dummy_test_func.called)
         result = self.dummy_test_func()
         self.assertTrue(result)
@@ -1323,7 +1322,7 @@ class RunOnceTests(test.NoDBTestCase):
         RunOnceTests.fake_logger.assert_called_once()
         self.assertIsNone(result)
 
-    def test_wrapped_funtions_called_once_raises(self):
+    def test_wrapped_functions_called_once_raises(self):
         self.assertFalse(self.dummy_test_func.called)
         self.assertRaises(ValueError, self.dummy_test_func, fail=True)
         self.assertTrue(self.dummy_test_func.called)
@@ -1334,7 +1333,7 @@ class RunOnceTests(test.NoDBTestCase):
         RunOnceTests.fake_logger.assert_called_once()
         self.assertIsNone(result)
 
-    def test_wrapped_funtions_can_be_reset(self):
+    def test_wrapped_functions_can_be_reset(self):
         # assert we start with a clean state
         self.assertFalse(self.dummy_test_func.called)
         result = self.dummy_test_func()
