@@ -533,6 +533,7 @@ class VMOpsTestCase(test_base.HyperVBaseTestCase):
         events = self._vmops._get_neutron_events(network_info)
         self.assertEqual([], events)
 
+    @mock.patch.object(vmops.version, 'product_string')
     @mock.patch.object(vmops.VMOps, '_attach_pci_devices')
     @mock.patch.object(vmops.VMOps, '_requires_secure_boot')
     @mock.patch.object(vmops.VMOps, '_requires_certificate')
@@ -554,6 +555,7 @@ class VMOpsTestCase(test_base.HyperVBaseTestCase):
                               mock_requires_certificate,
                               mock_requires_secure_boot,
                               mock_attach_pci_devices,
+                              mock_product_string,
                               enable_instance_metrics,
                               vm_gen=constants.VM_GEN_1,
                               vnuma_enabled=False,
@@ -604,7 +606,8 @@ class VMOpsTestCase(test_base.HyperVBaseTestCase):
             mock_instance.name, mock_instance.flavor.memory_mb, mem_per_numa,
             mock_instance.flavor.vcpus, cpus_per_numa,
             CONF.hyperv.limit_cpu_features, dynamic_memory_ratio,
-            host_shutdown_action=host_shutdown_action)
+            host_shutdown_action=host_shutdown_action,
+            chassis_asset_tag=mock_product_string.return_value)
 
         mock_configure_remotefx.assert_called_once_with(mock_instance, vm_gen)
         mock_create_scsi_ctrl = self._vmops._vmutils.create_scsi_controller
@@ -986,8 +989,7 @@ class VMOpsTestCase(test_base.HyperVBaseTestCase):
             mock_InstanceMetadata.assert_called_once_with(
                 mock_instance, content=[mock.sentinel.FILE],
                 extra_md={'admin_pass': mock.sentinel.PASSWORD},
-                network_info=mock.sentinel.NET_INFO,
-                request_context=self.context)
+                network_info=mock.sentinel.NET_INFO)
             mock_get_configdrive_path.assert_has_calls(
                 expected_get_configdrive_path_calls)
             mock_ConfigDriveBuilder.assert_called_with(
@@ -1372,12 +1374,10 @@ class VMOpsTestCase(test_base.HyperVBaseTestCase):
     def test_get_vm_state(self):
         summary_info = {'EnabledState': os_win_const.HYPERV_VM_STATE_DISABLED}
 
-        with mock.patch.object(self._vmops._vmutils,
-                               'get_vm_summary_info') as mock_get_summary_info:
-            mock_get_summary_info.return_value = summary_info
+        self._vmops._vmutils.get_vm_summary_info.return_value = summary_info
 
-            response = self._vmops._get_vm_state(mock.sentinel.FAKE_VM_NAME)
-            self.assertEqual(response, os_win_const.HYPERV_VM_STATE_DISABLED)
+        response = self._vmops._get_vm_state(mock.sentinel.FAKE_VM_NAME)
+        self.assertEqual(response, os_win_const.HYPERV_VM_STATE_DISABLED)
 
     @mock.patch.object(vmops.VMOps, '_get_vm_state')
     def test_wait_for_power_off_true(self, mock_get_state):
@@ -1416,12 +1416,11 @@ class VMOpsTestCase(test_base.HyperVBaseTestCase):
 
     def test_list_instance_uuids(self):
         fake_uuid = '4f54fb69-d3a2-45b7-bb9b-b6e6b3d893b3'
-        with mock.patch.object(self._vmops._vmutils,
-                               'list_instance_notes') as mock_list_notes:
-            mock_list_notes.return_value = [('fake_name', [fake_uuid])]
+        self._vmops._vmutils.list_instance_notes.return_value = (
+            [('fake_name', [fake_uuid])])
 
-            response = self._vmops.list_instance_uuids()
-            mock_list_notes.assert_called_once_with()
+        response = self._vmops.list_instance_uuids()
+        self._vmops._vmutils.list_instance_notes.assert_called_once_with()
 
         self.assertEqual(response, [fake_uuid])
 
