@@ -22,15 +22,15 @@ from os_vif.objects import fields as osv_fields
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_utils.fixture import uuidsentinel as uuids
-import six
 
 from nova import exception
 from nova.network import model as network_model
 from nova import objects
 from nova.pci import utils as pci_utils
 from nova import test
+from nova.tests import fixtures as nova_fixtures
+from nova.tests.fixtures import libvirt as fakelibvirt
 from nova.tests.unit.virt import fakelibosinfo
-from nova.tests.unit.virt.libvirt import fakelibvirt
 from nova.virt.libvirt import config as vconfig
 from nova.virt.libvirt import vif
 
@@ -150,7 +150,8 @@ class LibvirtVifTestCase(test.NoDBTestCase):
         type=network_model.VIF_TYPE_OVS,
         details={'port_filter': True},
         devname='tap-xxx-yyy-zzz',
-        ovs_interfaceid=uuids.ovs)
+        ovs_interfaceid=uuids.ovs,
+        delegate_create=True)
 
     vif_ovs_legacy = network_model.VIF(id=uuids.vif,
         address='ca:fe:de:ad:be:ef',
@@ -206,6 +207,18 @@ class LibvirtVifTestCase(test.NoDBTestCase):
                  'pci_slot': '0000:0a:00.1',
                  'physical_network': 'phynet1'})
 
+    vif_hw_veb_arq = network_model.VIF(id=uuids.vif,
+        address='ca:fe:de:ad:be:ef',
+        network=network_8021,
+        type=network_model.VIF_TYPE_HW_VEB,
+        vnic_type=network_model.VNIC_TYPE_ACCELERATOR_DIRECT,
+        ovs_interfaceid=None,
+        details={
+            network_model.VIF_DETAILS_VLAN: 100},
+        profile={'pci_vendor_info': '1137:0043',
+                 'pci_slot': '0000:0a:00.1',
+                 'physical_network': 'phynet1'})
+
     vif_hw_veb_trusted = network_model.VIF(id=uuids.vif,
         address='ca:fe:de:ad:be:ef',
         network=network_8021,
@@ -411,7 +424,8 @@ class LibvirtVifTestCase(test.NoDBTestCase):
 
         self.os_vif_ovs_prof = osv_objects.vif.VIFPortProfileOpenVSwitch(
             interface_id="07bd6cea-fb37-4594-b769-90fc51854ee9",
-            profile_id="fishfood")
+            profile_id="fishfood",
+            create_port=True)
 
         self.os_vif_repr_prof = osv_objects.vif.VIFPortProfileOVSRepresentor(
             interface_id="07bd6cea-fb37-4594-b769-90fc51854ee9",
@@ -502,11 +516,19 @@ class LibvirtVifTestCase(test.NoDBTestCase):
 
     def setUp(self):
         super(LibvirtVifTestCase, self).setUp()
-        self.useFixture(fakelibvirt.FakeLibvirtFixture(stub_os_vif=False))
+
+        self.libvirt = self.useFixture(
+            nova_fixtures.LibvirtFixture(stub_os_vif=False))
+
         # os_vif.initialize is typically done in nova-compute startup
         os_vif.initialize()
         self.setup_os_vif_objects()
 
+        # multiqueue configuration is host OS specific
+        self.mock_uname = self.libvirt.mock_uname
+        self.mock_uname.return_value = fakelibvirt.os_uname(
+            'Linux', '', '5.10.13-200-generic', '', 'x86_64')
+
     def _get_node(self, xml):
         doc = etree.fromstring(xml)
         ret = doc.findall('./devices/interface')
@@ -567,7 +589,7 @@ class LibvirtVifTestCase(test.NoDBTestCase):
         self.assertEqual(tx_queue_size, tx_want)
 
     def _assertXmlEqual(self, expectedXmlstr, actualXmlstr):
-        if not isinstance(actualXmlstr, six.string_types):
+        if not isinstance(actualXmlstr, str):
             actualXmlstr = etree.tostring(actualXmlstr, encoding='unicode',
                                           pretty_print=True)
         self.assertXmlEqual(expectedXmlstr, actualXmlstr)
@@ -641,30 +663,36 @@ class LibvirtVifTestCase(test.NoDBTestCase):
     def test_virtio_multiqueue(self):
         self._test_virtio_multiqueue(4, '4')
 
-    @mock.patch('os.uname', return_value=('Linux', '', '2.6.32-21-generic'))
-    def test_virtio_multiqueue_in_kernel_2(self, mock_uname):
+    def test_virtio_multiqueue_in_kernel_2(self):
+        self.mock_uname.return_value = fakelibvirt.os_uname(
+            'Linux', '', '2.6.32-21-generic', '', '')
         self._test_virtio_multiqueue(10, '1')
 
-    @mock.patch('os.uname', return_value=('Linux', '', '3.19.0-47-generic'))
-    def test_virtio_multiqueue_in_kernel_3(self, mock_uname):
+    def test_virtio_multiqueue_in_kernel_3(self):
+        self.mock_uname.return_value = fakelibvirt.os_uname(
+            'Linux', '', '3.19.0-47-generic', '', '')
         self._test_virtio_multiqueue(10, '8')
 
-    @mock.patch('os.uname', return_value=('Linux', '', '4.2.0-35-generic'))
-    def test_virtio_multiqueue_in_kernel_4(self, mock_uname):
+    def test_virtio_multiqueue_in_kernel_4(self):
+        self.mock_uname.return_value = fakelibvirt.os_uname(
+            'Linux', '', '4.2.0-35-generic', '', '')
         self._test_virtio_multiqueue(10, '10')
 
-    @mock.patch('os.uname', return_value=('Linux', '', '2.6.32-21-generic'))
-    def test_virtio_multiqueue_in_kernel_2_max_queues(self, mock_uname):
+    def test_virtio_multiqueue_in_kernel_2_max_queues(self):
+        self.mock_uname.return_value = fakelibvirt.os_uname(
+            'Linux', '', '2.6.32-21-generic', '', '')
         self.flags(max_queues=2, group='libvirt')
         self._test_virtio_multiqueue(10, '2')
 
-    @mock.patch('os.uname', return_value=('Linux', '', '3.19.0-47-generic'))
-    def test_virtio_multiqueue_in_kernel_3_max_queues(self, mock_uname):
+    def test_virtio_multiqueue_in_kernel_3_max_queues(self):
+        self.mock_uname.return_value = fakelibvirt.os_uname(
+            'Linux', '', '3.19.0-47-generic', '', '')
         self.flags(max_queues=2, group='libvirt')
         self._test_virtio_multiqueue(10, '2')
 
-    @mock.patch('os.uname', return_value=('Linux', '', '4.2.0-35-generic'))
-    def test_virtio_multiqueue_in_kernel_4_max_queues(self, mock_uname):
+    def test_virtio_multiqueue_in_kernel_4_max_queues(self):
+        self.mock_uname.return_value = fakelibvirt.os_uname(
+            'Linux', '', '4.2.0-35-generic', '', '')
         self.flags(max_queues=2, group='libvirt')
         self._test_virtio_multiqueue(10, '2')
 
@@ -852,7 +880,7 @@ class LibvirtVifTestCase(test.NoDBTestCase):
 
     @mock.patch.object(vif.designer, 'set_vif_guest_frontend_config',
                        wraps=vif.designer.set_vif_guest_frontend_config)
-    def test_model_sriov_direct(self, mock_set):
+    def _test_model_sriov(self, vinc_type, mock_set):
         """Direct attach vNICs shouldn't retrieve info from image_meta."""
         self.flags(use_virtio_for_bridges=True,
                    virt_type='kvm',
@@ -864,13 +892,19 @@ class LibvirtVifTestCase(test.NoDBTestCase):
         image_meta = objects.ImageMeta.from_dict(
             {'properties': {'hw_vif_model': 'virtio'}})
         conf = d.get_base_config(None, 'ca:fe:de:ad:be:ef', image_meta,
-                                 None, 'kvm', network_model.VNIC_TYPE_DIRECT)
+                                 None, 'kvm', vinc_type)
         mock_set.assert_called_once_with(mock.ANY, 'ca:fe:de:ad:be:ef',
                                          None, None, None, None)
         self.assertIsNone(conf.vhost_queues)
         self.assertIsNone(conf.driver_name)
         self.assertIsNone(conf.model)
 
+    def test_model_sriov_direct(self):
+        self._test_model_sriov(network_model.VNIC_TYPE_DIRECT)
+
+    def test_model_accelerator_direct(self):
+        self._test_model_sriov(network_model.VNIC_TYPE_ACCELERATOR_DIRECT)
+
     def test_model_qemu(self):
         vif_objs = [
             self.vif_bridge,
@@ -911,15 +945,6 @@ class LibvirtVifTestCase(test.NoDBTestCase):
 
             self._assertModel(xml, network_model.VIF_MODEL_VIRTIO, "qemu")
 
-    def test_model_xen(self):
-        self.flags(use_virtio_for_bridges=True,
-                   virt_type='xen',
-                   group='libvirt')
-
-        d = vif.LibvirtGenericVIFDriver()
-        xml = self._get_instance_xml(d, self.vif_bridge)
-        self._assertModel(xml)
-
     def test_generic_driver_none(self):
         d = vif.LibvirtGenericVIFDriver()
         self.assertRaises(exception.NovaException,
@@ -939,14 +964,9 @@ class LibvirtVifTestCase(test.NoDBTestCase):
                                   self.vif_bridge,
                                   self.vif_bridge['network']['bridge'])
 
-    @mock.patch.object(pci_utils, 'get_ifname_by_pci_address')
-    @mock.patch.object(pci_utils, 'get_vf_num_by_pci_address', return_value=1)
-    @mock.patch('nova.privsep.linux_net.set_device_macaddr')
-    @mock.patch('nova.privsep.linux_net.set_device_macaddr_and_vlan')
-    def _test_hw_veb_op(self, op, vlan, mock_set_macaddr_and_vlan,
-                        mock_set_macaddr, mock_get_vf_num,
-                        mock_get_ifname):
-        mock_get_ifname.side_effect = ['eth1', 'eth13']
+    def _test_hw_veb_op(self, op, vlan):
+        self.libvirt.mock_get_vf_num_by_pci_address.return_value = 1
+        pci_utils.get_ifname_by_pci_address.side_effect = ['eth1', 'eth13']
         vlan_id = int(vlan)
         port_state = 'up' if vlan_id > 0 else 'down'
         mac = ('00:00:00:00:00:00' if op.__name__ == 'unplug'
@@ -961,10 +981,13 @@ class LibvirtVifTestCase(test.NoDBTestCase):
             'set_macaddr': [mock.call('eth13', mac, port_state=port_state)]
         }
         op(self.instance, self.vif_hw_veb_macvtap)
-        mock_get_ifname.assert_has_calls(calls['get_ifname'])
-        mock_get_vf_num.assert_has_calls(calls['get_vf_num'])
-        mock_set_macaddr.assert_has_calls(calls['set_macaddr'])
-        mock_set_macaddr_and_vlan.assert_called_once_with(
+        pci_utils.get_ifname_by_pci_address.assert_has_calls(
+            calls['get_ifname'])
+        self.libvirt.mock_get_vf_num_by_pci_address.assert_has_calls(
+            calls['get_vf_num'])
+        self.libvirt.mock_set_device_macaddr.assert_has_calls(
+            calls['set_macaddr'])
+        self.libvirt.mock_set_device_macaddr_and_vlan.assert_called_once_with(
             'eth1', 1, mock.ANY, vlan_id)
 
     def test_plug_hw_veb(self):
@@ -1009,35 +1032,12 @@ class LibvirtVifTestCase(test.NoDBTestCase):
                        self.vif_iovisor['network']['id'],
                        self.instance.project_id)])
 
-    def _check_ovs_virtualport_driver(self, d, vif, want_iface_id):
-        xml = self._get_instance_xml(d, vif)
-        node = self._get_node(xml)
-        self._assertTypeAndMacEquals(node, "bridge", "source", "bridge",
-                                     vif, "br0")
-        vp = node.find("virtualport")
-        self.assertEqual(vp.get("type"), "openvswitch")
-        iface_id_found = False
-        for p_elem in vp.findall("parameters"):
-            iface_id = p_elem.get("interfaceid", None)
-            if iface_id:
-                self.assertEqual(iface_id, want_iface_id)
-                iface_id_found = True
-
-        self.assertTrue(iface_id_found)
-
-    def test_generic_ovs_virtualport_driver(self):
-        d = vif.LibvirtGenericVIFDriver()
-        want_iface_id = self.vif_ovs['ovs_interfaceid']
-        self._check_ovs_virtualport_driver(d,
-                                           self.vif_ovs,
-                                           want_iface_id)
-
     def test_direct_plug_with_port_filter_cap_no_nova_firewall(self):
         d = vif.LibvirtGenericVIFDriver()
         br_want = self.vif_midonet['devname']
         xml = self._get_instance_xml(d, self.vif_ovs_filter_cap)
         node = self._get_node(xml)
-        self._assertTypeAndMacEquals(node, "bridge", "target", "dev",
+        self._assertTypeAndMacEquals(node, "ethernet", "target", "dev",
                                      self.vif_ovs_filter_cap, br_want)
 
     def test_ib_hostdev_driver(self):
@@ -1073,7 +1073,7 @@ class LibvirtVifTestCase(test.NoDBTestCase):
         d1 = vif.LibvirtGenericVIFDriver()
         ins = objects.Instance(
             id=1, uuid='f0000000-0000-0000-0000-000000000001',
-            flavor=self.flavor_2vcpu,
+            image_ref=uuids.image_ref, flavor=self.flavor_2vcpu,
             project_id=723, system_metadata={}
         )
         d1.plug(ins, self.vif_tap)
@@ -1085,7 +1085,7 @@ class LibvirtVifTestCase(test.NoDBTestCase):
         d2 = vif.LibvirtGenericVIFDriver()
         mq_ins = objects.Instance(
             id=1, uuid='f0000000-0000-0000-0000-000000000001',
-            flavor=self.flavor_2vcpu,
+            image_ref=uuids.image_ref, flavor=self.flavor_2vcpu,
             project_id=723, system_metadata={
                 'image_hw_vif_multiqueue_enabled': 'True'
             }
@@ -1119,13 +1119,13 @@ class LibvirtVifTestCase(test.NoDBTestCase):
             self, mock_create_tap_dev, mock_set_mtu, mock_device_exists):
 
         self.flags(use_virtio_for_bridges=True,
-                   virt_type='xen',
+                   virt_type='lxc',
                    group='libvirt')
 
         d1 = vif.LibvirtGenericVIFDriver()
         ins = objects.Instance(
             id=1, uuid='f0000000-0000-0000-0000-000000000001',
-            flavor=self.flavor_2vcpu,
+            image_ref=uuids.image_ref, flavor=self.flavor_2vcpu,
             project_id=723, system_metadata={
                 'image_hw_vif_multiqueue_enabled': 'True'
             }
@@ -1143,7 +1143,7 @@ class LibvirtVifTestCase(test.NoDBTestCase):
         d1 = vif.LibvirtGenericVIFDriver()
         ins = objects.Instance(
             id=1, uuid='f0000000-0000-0000-0000-000000000001',
-            flavor=self.flavor_2vcpu,
+            image_ref=uuids.image_ref, flavor=self.flavor_2vcpu,
             project_id=723, system_metadata={
                 'image_hw_vif_multiqueue_enabled': 'True',
                 'image_hw_vif_model': 'e1000',
@@ -1195,9 +1195,8 @@ class LibvirtVifTestCase(test.NoDBTestCase):
         self.assertEqual(1, len(node))
         self._assertPciEqual(node, self.vif_hostdev_physical)
 
-    @mock.patch.object(pci_utils, 'get_ifname_by_pci_address',
-                       return_value='eth1')
-    def test_hw_veb_driver_macvtap(self, mock_get_ifname):
+    def test_hw_veb_driver_macvtap(self):
+        pci_utils.get_ifname_by_pci_address.return_value = 'eth1'
         d = vif.LibvirtGenericVIFDriver()
         xml = self._get_instance_xml(d, self.vif_hw_veb_macvtap)
         node = self._get_node(xml)
@@ -1239,9 +1238,20 @@ class LibvirtVifTestCase(test.NoDBTestCase):
                           self._get_instance_xml,
                           d,
                           self.vif_macvtap_exception)
-        self.assertIn('macvtap_source', six.text_type(e))
-        self.assertIn('macvtap_mode', six.text_type(e))
-        self.assertIn('physical_interface', six.text_type(e))
+        self.assertIn('macvtap_source', str(e))
+        self.assertIn('macvtap_mode', str(e))
+        self.assertIn('physical_interface', str(e))
+
+    def test_hw_veb_driver_arq(self):
+        d = vif.LibvirtGenericVIFDriver()
+        xml = self._get_instance_xml(d, self.vif_hw_veb_arq)
+        node = self._get_node(xml)
+        self._assertTypeAndPciEquals(node, "hostdev", self.vif_hw_veb_arq)
+        self._assertMacEquals(node, self.vif_hw_veb_arq)
+
+        conf = vconfig.LibvirtConfigGuestInterface()
+        conf.parse_dom(node)
+        self.assertEqual(conf.vlan, self.vif_hw_veb_arq["details"]["vlan"])
 
     @mock.patch('nova.virt.libvirt.vif.ensure_vlan')
     def test_macvtap_plug_vlan(self, ensure_vlan_mock):
@@ -1517,8 +1527,8 @@ class LibvirtVifTestCase(test.NoDBTestCase):
         d = vif.LibvirtGenericVIFDriver()
         xml = self._get_instance_xml(d, vif_model)
         node = self._get_node(xml)
-
-        self._assertXmlEqual(expected_xml, node)
+        node_xml = etree.tostring(node).decode()
+        self._assertXmlEqual(expected_xml, node_xml)
 
     def test_config_os_vif_bridge(self):
         os_vif_type = self.os_vif_bridge
@@ -1578,20 +1588,15 @@ class LibvirtVifTestCase(test.NoDBTestCase):
         vif_type = self.vif_agilio_ovs
 
         expected_xml = """
-            <interface type="bridge">
+            <interface type="ethernet">
              <mac address="22:52:25:62:e2:aa"/>
              <model type="virtio"/>
-             <source bridge="br0"/>
              <mtu size="9000"/>
              <target dev="nicdc065497-3c"/>
-             <virtualport type="openvswitch">
-              <parameters
-              interfaceid="07bd6cea-fb37-4594-b769-90fc51854ee9"/>
-             </virtualport>
-              <bandwidth>
-               <inbound average="100" peak="200" burst="300"/>
-               <outbound average="10" peak="20" burst="30"/>
-              </bandwidth>
+             <bandwidth>
+              <inbound average="100" peak="200" burst="300"/>
+              <outbound average="10" peak="20" burst="30"/>
+             </bandwidth>
             </interface>"""
 
         self._test_config_os_vif(os_vif_type, vif_type, expected_xml)
@@ -1631,15 +1636,11 @@ class LibvirtVifTestCase(test.NoDBTestCase):
         vif_type = self.vif_ovs
 
         expected_xml = """
-            <interface type="bridge">
+            <interface type="ethernet">
              <mac address="22:52:25:62:e2:aa"/>
              <model type="virtio"/>
-             <source bridge="br0"/>
              <mtu size="9000"/>
              <target dev="nicdc065497-3c"/>
-             <virtualport type="openvswitch">
-              <parameters interfaceid="07bd6cea-fb37-4594-b769-90fc51854ee9"/>
-             </virtualport>
              <bandwidth>
               <inbound average="100" peak="200" burst="300"/>
               <outbound average="10" peak="20" burst="30"/>
