@@ -28,7 +28,6 @@ from oslo_utils import units
 from oslo_utils import uuidutils
 from oslo_vmware import exceptions as vexc
 from oslo_vmware.objects import datastore as ds_obj
-import six
 
 from nova import exception
 from nova.virt.vmwareapi import constants
@@ -170,6 +169,7 @@ class ManagedObjectReference(object):
         # typically have values like vm-123 or
         # host-232 and not UUID.
         self.value = value
+        self._value_1 = value
         # Managed Object Reference type
         # attributes hold the name of the type
         # of the vCenter object the value
@@ -187,10 +187,10 @@ class ObjectContent(object):
     def __init__(self, obj_ref, prop_list=None, missing_list=None):
         self.obj = obj_ref
 
-        if not isinstance(prop_list, collections.Iterable):
+        if not isinstance(prop_list, collections.abc.Iterable):
             prop_list = []
 
-        if not isinstance(missing_list, collections.Iterable):
+        if not isinstance(missing_list, collections.abc.Iterable):
             missing_list = []
 
         # propSet is the name your Python code will need to
@@ -664,8 +664,8 @@ class ClusterComputeResource(ManagedObject):
             summary.numCpuCores += host_summary.hardware.numCpuCores
             summary.numCpuThreads += host_summary.hardware.numCpuThreads
             summary.totalMemory += host_summary.hardware.memorySize
-            free_memory = (host_summary.hardware.memorySize / units.Mi
-                           - host_summary.quickStats.overallMemoryUsage)
+            free_memory = (host_summary.hardware.memorySize / units.Mi -
+                           host_summary.quickStats.overallMemoryUsage)
             summary.effectiveMemory += free_memory if connected else 0
             summary.numEffectiveHosts += 1 if connected else 0
         self.set("summary", summary)
@@ -759,7 +759,8 @@ class HostSystem(ManagedObject):
 
         product = DataObject()
         product.name = "VMware ESXi"
-        product.version = constants.MIN_VC_VERSION
+        # Avoid deprecation warning
+        product.version = constants.NEXT_MIN_VC_VERSION
         config = DataObject()
         config.product = product
         summary.config = config
@@ -773,6 +774,7 @@ class HostSystem(ManagedObject):
         self.set("capability.maxHostSupportedVcpus", 600)
         self.set("summary.hardware", hardware)
         self.set("summary.runtime", runtime)
+        self.set("summary.quickStats", quickstats)
         self.set("config.network.pnic", net_info_pnic)
         self.set("connected", connected)
 
@@ -1008,7 +1010,7 @@ def create_vm(uuid=None, name=None,
     if vm_path.rel_path == '':
         vm_path = vm_path.join(name, name + '.vmx')
 
-    for key, value in six.iteritems(_db_content["Datastore"]):
+    for key, value in _db_content["Datastore"].items():
         if value.get('summary.name') == vm_path.datastore:
             ds = key
             break
@@ -1101,7 +1103,7 @@ def fake_fetch_image(context, instance, host, port, dc_name, ds_name,
 def _get_vm_mdo(vm_ref):
     """Gets the Virtual Machine with the ref from the db."""
     if _db_content.get("VirtualMachine", None) is None:
-            raise exception.NotFound("There is no VM registered")
+        raise exception.NotFound("There is no VM registered")
     if vm_ref not in _db_content.get("VirtualMachine"):
         raise exception.NotFound("Virtual Machine with ref %s is not "
                                  "there" % vm_ref)
@@ -1224,12 +1226,7 @@ class FakeVim(object):
         self.client = FakeClient()
         self.client.factory = FakeFactory()
 
-        transport = DataObject()
-        transport.cookiejar = "Fake-CookieJar"
-        options = DataObject()
-        options.transport = transport
-
-        self.client.options = options
+        self.client.cookiejar = "Fake-CookieJar"
 
         service_content = self.client.factory.create('ns0:ServiceContent')
         service_content.propertyCollector = "PropCollector"
@@ -1242,7 +1239,8 @@ class FakeVim(object):
 
         about_info = DataObject()
         about_info.name = "VMware vCenter Server"
-        about_info.version = "5.1.0"
+        # Avoid deprecation warning
+        about_info.version = constants.NEXT_MIN_VC_VERSION
         about_info.instanceUuid = _FAKE_VCENTER_UUID
 
         service_content.about = about_info
