@@ -29,6 +29,7 @@ import shutil
 import tempfile
 
 import eventlet
+from eventlet import tpool
 from keystoneauth1 import loading as ks_loading
 import netaddr
 from openstack import connection
@@ -43,10 +44,9 @@ import oslo_messaging as messaging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import importutils
+from oslo_utils.secretutils import md5
 from oslo_utils import strutils
 from oslo_utils import timeutils
-import six
-from six.moves import range
 
 import nova.conf
 from nova import exception
@@ -81,12 +81,6 @@ _FILE_CACHE = {}
 _SERVICE_TYPES = service_types.ServiceTypes()
 
 
-if hasattr(inspect, 'getfullargspec'):
-    getargspec = inspect.getfullargspec
-else:
-    getargspec = inspect.getargspec
-
-
 # NOTE(mikal): this seems to have to stay for now to handle os-brick
 # requirements. This makes me a sad panda.
 def get_root_helper():
@@ -256,11 +250,11 @@ def utf8(value):
     http://github.com/facebook/tornado/blob/master/tornado/escape.py
 
     """
-    if value is None or isinstance(value, six.binary_type):
+    if value is None or isinstance(value, bytes):
         return value
 
-    if not isinstance(value, six.text_type):
-        value = six.text_type(value)
+    if not isinstance(value, str):
+        value = str(value)
 
     return value.encode('utf-8')
 
@@ -373,7 +367,7 @@ def sanitize_hostname(hostname, default_name=None):
         hostname = hostname.encode('latin-1', 'ignore').decode('latin-1')
 
     hostname = truncate_hostname(hostname)
-    hostname = re.sub('[ _]', '-', hostname)
+    hostname = re.sub(r'[ _\.]', '-', hostname)
     hostname = re.sub(r'[^\w.-]+', '', hostname)
     hostname = hostname.lower()
     hostname = hostname.strip('.-')
@@ -554,7 +548,7 @@ def expects_func_args(*args):
         @functools.wraps(dec)
         def _decorator(f):
             base_f = safe_utils.get_wrapped_function(f)
-            argspec = getargspec(base_f)
+            argspec = inspect.getfullargspec(base_f)
             if argspec[1] or argspec[2] or set(args) <= set(argspec[0]):
                 # NOTE (ndipanov): We can't really tell if correct stuff will
                 # be passed if it's a function with *args or **kwargs so
@@ -618,7 +612,7 @@ def validate_integer(value, name, min_value=None, max_value=None):
     try:
         return strutils.validate_integer(value, name, min_value, max_value)
     except ValueError as e:
-        raise exception.InvalidInput(reason=six.text_type(e))
+        raise exception.InvalidInput(reason=str(e))
 
 
 def _serialize_profile_info():
@@ -691,13 +685,13 @@ def spawn_n(func, *args, **kwargs):
 
 def tpool_execute(func, *args, **kwargs):
     """Run func in a native thread"""
-    eventlet.tpool.execute(func, *args, **kwargs)
+    tpool.execute(func, *args, **kwargs)
 
 
 def is_none_string(val):
     """Check if a string represents a None value.
     """
-    if not isinstance(val, six.string_types):
+    if not isinstance(val, str):
         return False
 
     return val.lower() == 'none'
@@ -730,7 +724,7 @@ def get_system_metadata_from_image(image_meta, flavor=None):
         if key in SM_SKIP_KEYS:
             continue
 
-        new_value = safe_truncate(six.text_type(value), 255)
+        new_value = safe_truncate(str(value), 255)
         system_meta[prefix_format % key] = new_value
 
     for key in SM_INHERITABLE_KEYS:
@@ -784,9 +778,9 @@ def get_hash_str(base_str):
 
     If base_str is a Unicode string, encode it to UTF-8.
     """
-    if isinstance(base_str, six.text_type):
+    if isinstance(base_str, str):
         base_str = base_str.encode('utf-8')
-    return hashlib.md5(base_str).hexdigest()
+    return md5(base_str, usedforsecurity=False).hexdigest()
 
 
 def get_sha256_str(base_str):
@@ -797,7 +791,7 @@ def get_sha256_str(base_str):
     or anything else that needs to be retained for a long period a salted
     hash is better.
     """
-    if isinstance(base_str, six.text_type):
+    if isinstance(base_str, str):
         base_str = base_str.encode('utf-8')
     return hashlib.sha256(base_str).hexdigest()
 
@@ -986,7 +980,7 @@ def get_sdk_adapter(service_type, check_service=False):
     except sdk_exc.ServiceDiscoveryException as e:
         raise exception.ServiceUnavailable(
             _("The %(service_type)s service is unavailable: %(error)s") %
-            {'service_type': service_type, 'error': six.text_type(e)})
+            {'service_type': service_type, 'error': str(e)})
     return getattr(conn, service_type)
 
 
@@ -1069,8 +1063,8 @@ def raise_if_old_compute():
                 ctxt, ['nova-compute'])
         except exception.DBNotAllowed:
             # This most likely means we are in a nova-compute service
-            # which is configured with a connection to the API database.
-            # We should not be attempting to "get out" of our cell to
+            # configured which is configured with a connection to the API
+            # database. We should not be attempting to "get out" of our cell to
             # look at the minimum versions of nova-compute services in other
             # cells, so DBNotAllowed was raised. Leave a warning message
             # and fall back to only querying computes in our cell.
